library(log4r)
logfile <- "/esarchive/scratch/aho/package_installation_logfile.txt"
my_console_appender <- console_appender(layout = default_log_layout())
my_file_appender <- file_appender(logfile, append = TRUE, 
                            layout = default_log_layout())

my_logger <- log4r::logger(threshold = "INFO", 
                appenders= list(my_console_appender,my_file_appender))


#======== Save all package info from Machine A =============
# Only the shared path
ip <- installed.packages(lib.loc = .libPaths()[2])
# Only save the useful info
ip <- ip[, c(1, 3, 5, 6, 8, 9, 10)]

R_version <- strsplit(R.Version()$version.string, ' ')[[1]][3]
machine <- 'WS' #'Nord4'
save_filename <- paste0(paste0("/esarchive/scratch/aho/R_packages_version/packages_in_", machine, "_", R_version))

saveRDS(ip, paste0(save_filename, '_', format(Sys.Date(), '%Y%m%d'), ".rds"))
write.csv(ip, paste0(save_filename, '_', format(Sys.Date(), '%Y%m%d'), ".csv"))

#============================================================


# Get the package names from Nord4, WS, and current 4.1.2
#ip <- readRDS(paste0(save_filename, ".rds"))
nord4 <- readRDS("/esarchive/scratch/aho/R_packages_version/packages_in_Nord4_4.1.2_20221110.rds")
nord4_name <- as.vector(nord4[, 1])

ws <- readRDS("/esarchive/scratch/aho/R_packages_version/packages_in_WS_4.1.2_20221110.rds")
ws_name <- as.vector(ws[, 1])

nord4new <- readRDS("/esarchive/scratch/aho/R_packages_version/packages_in_Nord4_4.1.2_20220513.rds")
nord4new_name <- as.vector(nord4new[, 1])


current_packages <- installed.packages(lib.loc = .libPaths()[1])[, 1]
current_packages <- as.vector(current_packages)

# Only install the packages in both machines
packages_to_install <- intersect(nord4new_name, ws_name)

packages_to_install <- union(nord4_name, ws_name)

extra_pkg <- nord4new_name[which(!nord4new_name %in% packages_to_install)]
missing_pkg <- packages_to_install[!packages_to_install %in% nord4new_name]

# Install the packages in WS but not hub
packages_to_install <- ws_name[!ws_name %in% current_packages]


#START FROM 1
start <- 61
end <- length(packages_to_install)
for (ii in packages_to_install[start:end]) {
  if (!ii %in% current_packages) {
#    log4r::info(my_logger, paste0('Start installing "', ii, '"...'))
    print(paste0('Start installing "', ii, '"...'))
    install.packages(ii)
#    library(ii, character.only = TRUE)
  } else {
#    log4r::info(my_logger, paste0('Package "', ii, '" is installed already.'))
    print(paste0('Package "', ii, '" is installed already.'))
#    library(ii, character.only = TRUE)
  }
}

# To see if packages are installed successfully
invisible(lapply(packages_to_install[start:end], library, character.only = TRUE))
which(!packages_to_install[start:end] %in% names(sessionInfo()$otherPkgs))

# unload all the packages
invisible(lapply(paste0('package:', names(sessionInfo()$otherPkgs)), detach, character.only=TRUE, unload=TRUE))


#========================================
new <- readRDS("/esarchive/scratch/aho/R_packages_version/packages_in_Nord4_4.1.2.rds")
new_name <- as.vector(new[, 1])

for (ii in 1:length(new_name)) {
  if (!new_name[ii] %in% sessionInfo()$basePkgs) {
    print(paste0('Loading ', ii, 'th package: ', new_name[ii]))
    invisible(lapply(new_name[ii], library, character.only = TRUE))

    invisible(lapply(paste0('package:', names(sessionInfo()$otherPkgs)), detach, character.only=TRUE, unload=TRUE))
  }
}
