library(clock)
library(lubridate)

# 1. Create/Parse a date
## base
as.Date("2019-01-31")
as.POSIXct("2017/11/28", tz = 'UTC')

## lubridate
ymd("2019-01-31")
ymd("2019-01-31", "20190131")
ymd("2020 July 31")
mdy("July 31 2020")

## clock
date_build(2019, 1, 31)
date_parse("January 5, 2020", format = "%B %d, %Y")
date_parse(c("2020/1/5", "10-03-05", "2020/2/2"), format = c("%Y/%m/%d", "%y-%m-%d"))

# 2. Create a date with time
## base
as.POSIXct('2019-01-31 12:30', tz = 'UTC')

## lubridate
ymd_hms("2019-01-31 12:30:00", tz = 'UTC')

## clock
date_time_build(2019, 1, 1, 12, 30, zone = 'UTC')

# 3. Create a sequence of dates
## base / lubridate
seq(ymd('2019-01-31'), ymd('2019-12-31'), by = 'months')

## clock
date_build(2019, 1:12, "last")
date_build(2019, 1:12, 31, invalid = "previous")
seq(date_build(2019, 1, 20), date_build(2019, 2, 5), by = 1)

# 4. Rounding
# months
x <- seq(ymd('2019-01-15'), ymd('2019-12-15'), by = '30 days')
# lubridate
floor_date(x, unit = 'month')

# weeks
x <- seq(ymd('2019-01-15'), ymd('2019-02-15'), by = 'days')
# lubridate
floor_date(x, unit = 'week')  # Sunday is the first day
# clock
date_floor(x, "week", n = 1, origin = date_build(2019, 1, 13))

# 5. Shifting weekdays
x <- date_build(2020, 1, 1:2)
as_weekday(x)
tue <- weekday(clock_weekdays$tuesday)
date_shift(x, target = tue)
date_shift(x, target = tue, which = 'previous')

# 6. Get and set components
x <- date_time_build(2020, 1, 1:2, 12, 30, zone = 'UTC')
# lubridate
year(x); year(x) <- 2019
month(x)
day(x)
wday(x)
hour(x)
tz(x)

# clock
get_year(x); set_year(x, 2021)
get_month(x)
get_day(x)
get_hour(x)
date_zone(x)

# 7. Calendars class
x <- date_build(2020, 3, 31)
as_year_month_day(x)
as_year_month_weekday(x)
as_year_day(x)
as_year_quarter_day(x)
as_iso_year_week_day(x)

add_months(x, 1)
add_months(x, 1, invalid = "previous")
## Create non-existing day
as_year_month_day(x) + duration_months(1)





