library(startR)
library(s2dv)

path_exp <- "/esarchive/exp/ecearth/a3vg/diags/DCPP/EC-Earth-Consortium/EC-Earth3-HR/dcppA-hindcast/$memb$/Omon/$var$/r360x180/v*/$var$_Omon_EC-Earth3-HR_dcppA-hindcast_s$sdate$-$memb$_r360x180_$chunk$12.nc"

membs <- paste0('r', 16:30, 'i1p1f1')
lat_min <- 10
lat_max <- 20
lon_min <- -10
lon_max <- 10
grid_target <- "/esarchive/scratch/Earth/acarreri/fx/interpolation/earthdiagnostics_interpolation_EC-Earth_r360x180.nc"
interp_method <- 'bil'
yr1 <- 1970
yr2 <- 2014

    data <- Start(dat = path_exp,
                      var = 'tos',
                      memb = membs,
                      sdate = paste(yr1:yr2),
                      time = c(1:8),
                      chunk = 'all',
                      chunk_depends = 'sdate',
                      time_across = 'chunk', 
                      merge_across_dims = TRUE,
                      lat = values(list(lat_min, lat_max)),
                      lon = values(list(lon_min, lon_max)),
                      transform = CDORemapper,
                      transform_extra_cells = 8,
                      transform_params = list(grid = grid_target,
                                              method = interp_method),
                      transform_vars = c('lat', 'lon'),
                      lat_reorder = Sort(decreasing = FALSE),
                      lon_reorder = CircularSort(-180, 180),
                      synonims = list(lon = c('lon', 'longitude'),
                                      lat = c('lat', 'latitude')),
                      return_vars = list(lat = NULL, lon = NULL), #list(time= c('sdate','chunk'), 
                      num_procs = 8, 
                      retrieve = FALSE)

func <- function(data, first_month, last_month, init_month, cores, opt) {
   # data: [dat, time, sdate, memb]
  if (opt == 1) {
    res <- s2dv::Season(data, time_dim = names(dim(data))[2], monini = init_month, moninf = first_month, monsup = last_month, ncores = cores)
  } else if (opt == 2) {
    moninf <- first_month - init_month + 1
    monsup <- last_month - init_month + 1
    moninf <- ifelse(moninf <= 0, moninf + 12, moninf)
    monsup <- ifelse(monsup <= 0, monsup + 12, monsup)
    while (monsup < moninf)  monsup <- monsup + 12
    res <- apply(data, c(1, 3, 4), s2dv:::.Season, monini = init_month, moninf = moninf, monsup = monsup)
   if (length(dim(res)) != 4) dim(res) <- c(time = 1, dim(res))
  } else if (opt == 3) {
    data <- ClimProjDiags::Subset(data, 'time', list(3:8))
    res <- s2dv::MeanDims(data, 'time', drop = F)
  }
  res <- s2dv::Reorder(res, c('dat', 'time', 'sdate', 'memb'))
  return(res)
}

step <- Step(func, target_dims = c('dat', 'time', 'sdate', 'memb'),
             output_dims = c('dat', 'time', 'sdate', 'memb'))

#wf <- AddStep(data, step, first_month = 1, last_month = 12, init_month = 11,
#              cores = 1, opt = 1)


library(rbenchmark)

benchmark(
  "Season, 1 core" = {
    wf <- AddStep(data, step, first_month = 1, last_month = 12, init_month = 11,
                  cores = 1, opt = 1)
    suppressWarnings(
    output <- Compute(wf, chunks = list(lat = 1))$output1
    )
  },
  "Season, 16 cores" = {
    wf <- AddStep(data, step, first_month = 1, last_month = 12, init_month = 11,
                  cores = 16, opt = 1)
    suppressWarnings(
    output <- Compute(wf, chunks = list(lat = 1))$output1
    )
  },
  "MeanDims" = {
    wf <- AddStep(data, step, first_month = 1, last_month = 12, init_month = 11,
                  cores = 1, opt = 3)
    suppressWarnings(
    output <- Compute(wf, chunks = list(lat = 1))$output1
    )
  },
#  ".Season + for loop" = {
#    wf <- AddStep(data, step, first_month = 1, last_month = 12, init_month = 11,
#                  cores = 1, opt = 2)
#    output <- Compute(wf, chunks = list(lat = 1))$output1
#  },

  replications = 2,
  columns = c("test", "replications", "elapsed",
              "relative", "user.self", "sys.self")
)

              test replications elapsed relative user.self sys.self
3         MeanDims            2  74.423    1.000    17.212    0.367
1   Season, 1 core            2  84.922    1.141    28.615    0.312
2 Season, 16 cores            2 105.874    1.423    20.770   23.588
