library(sf)
library(ggplot2)
library(rnaturalearth)
library(RColorBrewer)
library(cowplot) # to add s2dv legend

library(startR)
repos <- "/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc"
data <- Start(dat = list(list(name = 'system5c3s', path = repos)),
              var = 'tas',
              sdate = '20170101',
              ensemble = indices(1),
              time = indices(1),
              lon = 'all', #values(list(-80, 40)), 
              lon_reorder = CircularSort(-180, 180),
              lat = 'all', #values(list(20, 80)), #'all',
              lat_reorder = Sort(decreasing = T),
              synonims = list(lat = c('lat', 'latitude'),
                              lon = c('lon', 'longitude')),
              return_vars = list(time = 'sdate',
                                 longitude = 'dat',
                                 latitude = 'dat'),
              metadata_dims = 'dat',  # it can be omitted since it is automatically specified as 'dat'
              retrieve = T)
lon <- attributes(data)$Variable$system5c3s$lon
lat <- attributes(data)$Variable$system5c3s$lat
data <- data - 273.15
mask <- drop(data)
mask[which(mask > -20 & mask < -10)] <- 0
mask[which(mask != 0)] <- 1

#==============================================================
# brks to have exceeded values
brks1 <- seq(-40, 40, 10)
# brks to cover all values
#brks1 <- seq(-ceiling(max(abs(range(data)))), ceiling(max(abs(range(data)))), length.out = 11)
#cols1 <- brewer.pal(9, 'PuOr')[-9]

tar_proj <- c(Robinson = 54030, Mollweide = 54009, "Lambert Europe" = 102014, Arctic = 3995)

#-------------------
# Crop EU region
#-------------------
data_eu <- ClimProjDiags::Subset(data, c('lat', 'lon'), list(21:61, 161:221))
lon_eu <- lon[161:221]
lat_eu <- lat[21:61]
mask_eu <- ClimProjDiags::Subset(mask, c('lat', 'lon'), list(21:61, 161:221))
# brks to cover all values
brks_eu <- seq(-ceiling(max(abs(range(data_eu)))), ceiling(max(abs(range(data_eu)))), length.out = 11)
# brks to have exceeded values
brks_eu <- seq(-18, 18, length.out = 11)

#==============================================================
# legend = NULL
PlotRobinson(data, lon, lat, brks = brks1, cols = NULL, legend = NULL, 
             toptitle = "PlotRobinson1", caption = "caption1",
             width = 8, height = 4, target_proj = tar_proj[1],
             dots = mask)

# legend = 'ggplot2'
PlotRobinson(data, lon, lat, brks = brks1, cols = NULL, legend = 'ggplot2',
             toptitle = "PlotRobinson1", caption = "caption1",
             dots = mask, colNA = 'red',
#             fileout = "~/tmp/PlotRobinson_ggplot2.png",
             width = 8, height = 4,
#             triangle_ends = c(F, F), bar_extra_margin = c(5, 0, 5, 0), 
 #            col_inf = 'red', col_sup = 'blue', 
             margin_scale = rep(0.5, 4))


# legend = 's2dv'
PlotRobinson(data, lon, lat, brks = brks1, cols = NULL, legend = 's2dv', 
             target_proj = "+proj=robin", #"ESRI:54030",
             toptitle = "PlotRobinson1", #caption = "caption1",
             dots = mask, units = "K", colNA = 'red',
#             fileout = "/esarchive/scratch/aho/tmp/PlotRobinson_s2dv_poly.png", 
             width = 8, height = 4, #6, #height can reduce the space btwn plot and colorbar
             triangle_ends = c(T, F), bar_extra_margin = c(0, 5, 0, 5),
#             col_inf = 'red', col_sup = 'blue',
             vertical = F, style = 'point')

PlotRobinson(data_eu, lon_eu, lat_eu, brks = brks_eu, cols = NULL, legend = 's2dv',
             dots = mask_eu, #dots_size = 4,
#             mask = mask_eu,
             toptitle = "Europe", units = "K", 
             target_proj = "+proj=lcc +lat_0=30 +lon_0=10 +lat_1=43 +lat_2=62 +x_0=0 +y_0=0 +ellps=intl +units=m +no_defs +type=crs", #"ESRI:102014", #tar_proj[3],
             width = 8, height = 5, data_size = 1.7,
             crop_coastlines = c(lonmin = -20, lonmax = 40, latmin = 28, latmax = 70),
             triangle_ends = c(T, T), bar_extra_margin = c(0.4, 5, 0.4, 5),
             vertical = F, style = 'point')

# PlotEquiMap
PlotEquiMap(data, lon = lon, lat = lat, brks = brks1, cols = NULL, 
             toptitle = "PlotEquiMap", filled.continent = F,
             dots = mask, units = "K", colNA = 'red',
             width = 8, height = 4)
#             triangle_ends = c(T, T), 
#             col_inf = 'red', col_sup = 'blue',

#---------------------------------------------

#-------------------
# Crop North Pole region
#-------------------
data_np <- ClimProjDiags::Subset(data, c('lat'), list(1:31))
lon_np <- lon
lat_np <- lat[1:31]
mask_np <- ClimProjDiags::Subset(mask, c('lat'), list(1:31))
# brks to cover all values
brks_np <- seq(-ceiling(max(abs(range(data_np)))), ceiling(max(abs(range(data_np)))), length.out = 11)
# brks to have exceeded values
brks_np <- seq(-40, 40, length.out = 11)


PlotRobinson(data_np, lon_np, lat_np, brks = brks_np, cols = NULL, legend = 's2dv',
             dots = mask_np, size = 1, dots_size = 0.01,
             toptitle = "North Pole", units = "K", target_proj = "+proj=stere", #tar_proj[4],
             width = 8, height = 8, #data_size = 1.6,
#             crop_coastlines = c(lonmin = -20, lonmax = 40, latmin = 28, latmax = 70),
             triangle_ends = c(T, T), bar_extra_margin = c(0.4, 5, 0.4, 5),
             vertical = F)

#PlotStereoMap
PlotStereoMap(data, lon = lon, lat = lat, brks = brks1, cols = NULL,
              toptitle = "PlotStereoMap", filled.continent = F,
              dots = mask, units = "K", colNA = 'red',
              fileout = '~/tmp/PlotStereoMap_3.png',
              width = 8, height = 4, latlim = c(-90, 90))

#--------------------------------------------

data2 <- readRDS("/esarchive/scratch/nperez/git/s2s-suite/Test_NAOmodule.RDS")
hcst <- data2$hcst$data[1, 1, 1, 1, 1, 1, , , 1]
lon2 <- c(data2$hcst$coords$lon)
lat2 <- c(data2$hcst$coords$lat)

brks2 <- seq(-1, 1, 0.2)
cols2 <- brewer.pal(10, 'PuOr')

PlotRobinson(hcst, lon2, lat2, brks = brks2, cols = cols2, legend = 's2dv',
             target_proj = 3995, crop_coastlines = c(lonmin = -80, lonmax = 40, latmin = 10, latmax = 80),
             toptitle = "Plot Region", #caption = "caption1",
             dots = NULL, units = "Pa",
#             fileout = "~/tmp/PlotRobinson_s2dv.png", 
             width = 8, height = 4,
             triangle_ends = c(T, T), bar_extra_margin = c(0, 5, 0, 5),
             col_inf = 'red', col_sup = 'blue',
             vertical = F)



