
library(s2dverification)
library(s2dv)

  ecearth <- list(name = 'm04o',
                  path = file.path('/esarchive/exp/ecearth',
                                   '/$EXP_NAME$/monthly_mean',
                                   '/$VAR_NAME$_f6h/$VAR_NAME$_$START_DATE$.nc'))

  ecmwf <- list(name = 'system5c3s',
                path = file.path('/esarchive/exp/ecmwf',
                                 '/$EXP_NAME$/monthly_mean',
                                 '/$VAR_NAME$_f6h/$VAR_NAME$_$START_DATE$.nc'))

  era5 <- list(name = 'era5',
               path = file.path('/esarchive/recon/ecmwf',
                                '$OBS_NAME$/monthly_mean',
                                '$VAR_NAME$_f1h/$VAR_NAME$_$YEAR$$MONTH$.nc'))

  startDates <- c(paste0(1981:2000,'1101'))
  startDates <- format(seq(as.POSIXct('1981-01-01'),as.POSIXct('1983-12-01'),by = 'months'), '%Y%m%d')
  time_length <- 4
  lon <- c(-20, 20)
  lat <- c(-50, 50)

  data <- Load(var = 'psl',
               exp = list(ecearth, ecmwf),
               obs = list(era5),
               sdates = startDates,
               storefreq = "monthly",
               leadtimemax = time_length,
               output = 'lonlat',
               lonmin = lon[1], lonmax = lon[2],
               latmin = lat[1], latmax = lat[2])

library(pryr)
print('memory_used')
print(mem_used())

# s2dverification
gc(reset = T)
start <- Sys.time()
mem1 <- gc()
res1 <- s2dverification::Clim(data$mod, data$obs)
mem2 <- gc()
end <- Sys.time()
total1 <- end - start
print(total1)
print(mem2 - mem1)


# s2dv
gc(reset = T)
start <- Sys.time()
mem1 <- gc()
res2 <- Clim(data$mod, data$obs)
mem2 <- gc()
end <- Sys.time()
total2 <- end - start
print(total2)
print(mem2 - mem1)


source('~/Clim_clim.R')
gc(reset = T)
start <- Sys.time()
mem1 <- gc()
res2 <- Clim_clim(data$mod, data$obs)
mem2 <- gc()
end <- Sys.time()
total2 <- end - start
print(total2)
print(mem2 - mem1)


source('~/Clim_for.R')

#======================================
benchmark("s2dverification::Clim" = {
            res_s2dverification <- s2dverification::Clim(data$mod, data$obs)
          },

          "s2dv::Clim" = {
             res_s2dv <- s2dv::Clim(data$mod, data$obs, ncores = 4)
          },

          "Clim_clim" = {
             res_s2dv_clim <- Clim_clim(data$mod, data$obs, ncores = 4)
          },

          "Clim_for" = {
             res_s2dv_clim_for <- Clim_for(data$mod, data$obs, ncores = 4)
          },

          replications = 10,
          columns = c("test", "replications", "elapsed",
                      "relative", "user.self", "sys.self"))

#                  test replications elapsed relative user.self sys.self
#3             Clim_clim           10  65.296    3.862    61.745    1.440
#2            s2dv::Clim           10  66.520    3.934    62.556    1.327
#1 s2dverification::Clim           10  16.907    1.000    16.205    0.700


#        test replications elapsed relative user.self sys.self
#2  Clim_clim           10  62.345    1.000    59.203    0.952
#3   Clim_for           10  67.888    1.089    67.496    0.384
#1 s2dv::Clim           10  62.352    1.000    59.458    0.852

