library(easyNCDF)
library(maps)
# ref.file: The netCDF file to provide the reference grid points
# countries: 'europe', 'all'
# lat_dim: 'latitude' for example
# lon_dim: 'longitude' for example
# savefile: If NULL, return an array
#ref.file <- '/esarchive/exp/ecmwf/system5c3s/monthly_mean/tas_f6h/tas_20170201.nc'
#ref.file <- '/esarchive/exp/ecmwf/s2s-monthly_ensfor/weekly_mean/tas_f6h/tas_20191212.nc'

create_mask <- function(ref.file, countries = 'europe', pacific_centered = FALSE, lat_dim = NULL, lon_dim = NULL, savefile = NULL) {

if (is.null(lat_dim) | is.null(lon_dim)) {
  var_names <- NcReadVarNames(ref.file)
  lat_dim <- var_names[which(var_names %in% s2dv:::.KnownLatNames())]
  lon_dim <- var_names[which(var_names %in% s2dv:::.KnownLonNames())]
}
latlon <- NcToArray(ref.file, vars_to_read = c(lat_dim, lon_dim))
lat <- NcToArray(ref.file, vars_to_read = lat_dim)[1, ]
lon <- NcToArray(ref.file, vars_to_read = lon_dim)[1, ]

# Turn longitude to [-180, 180] if pacific_centered is FALSE; [0, 360] if it's TRUE.
if (pacific_centered & any(lon < 0)) { #[-180, 180] --> [0, 360]
  lon <- ifelse(lon < 0, lon + 360, lon)
  lon <- sort(lon)
}
if (!pacific_centered & any(lon >= 180)) { #[0, 360] --> [-180, 180]
  lon <- ifelse(lon >= 180, lon - 360, lon)
  lon <- sort(lon)
}
#lon <- ((lon + 180) %% 360) - 180

#country <- matrix(list(), nrow = length(lat), ncol = length(lon))
#for (lat_i in 1:length(lat)) {
#  for (lon_i in 1:length(lon)) {
#    country[lat_i, lon_i] <- map.where(database = "world", lon[lon_i], lat[lat_i])
#  }
#}

country <- vector('list', length = length(lat))
#NOTE: Not sure if it is correct or not
world <- ifelse(pacific_centered, 'world2', 'world') # 'world2' is [0, 360]
for (lat_i in 1:length(lat)) {
  country[[lat_i]] <- map.where(database = world, x = lon, y = rep(lat[lat_i], length(lon)))
}
country <- array(unlist(country), dim = c(lon = length(lon), lat = length(lat)))

if (countries == 'europe') {
  countries <- c('Austria', 'Belgium', 'Bulgaria', 'Croatia', 'Cyprus', 'Czech Republic', 'Denmark', 'Estonia', 'Finland', 'France', 'Germany', 'Greece', 'Hungary', 'Ireland', 'Italy', 'Latvia', 'Lithuania', 'Netherlands', 'Poland', 'Portugal', 'Romania', 'Slovakia', 'Slovenia', 'Spain', 'Sweden', 'UK:Great Britain', 'Norway', 'Iceland', 'Switzerland','Bosnia and Herzegovina',  'Serbia', 'Montenegro', 'Macedonia', 'Albania', 'Turkey', 'Moldova', 'Belarus', 'Ukraine')
  countries.iso <- c('AUT', 'BEL', 'BGR', 'HRV', 'CYP', 'CZE', 'DNK', 'EST', 'FIN', 'FRA', 'DEU', 'GRC', 'HUN', 'IRL', 'ITA', 'LVA', 'LTU', 'NLD', 'POL', 'PRT', 'ROU', 'SVK', 'SVN', 'ESP', 'SWE', 'GBR', 'NOR', 'ISL', 'CHE', 'BIH', 'SRB', 'MNE', 'MKD', 'ALB', 'TUR', 'MDA', 'BLR', 'UKR')
} else if (countries == 'all') {
  countries <- unique(as.vector(countries))
  countries <- countries[which(!is.na(countries))]
}

mask <- array(0, dim = c(country = length(countries), lon = length(lon), lat = length(lat)))

for (nn in 1:length(countries)) {
  index <- which(country == countries[nn], arr.ind = TRUE)
  mask[nn, index[, 'row'], index[ ,'col']] <- 1
}

# Attributes
attr(mask, lon_dim) <- lon
attr(mask, lat_dim) <- lat
attr(mask, 'countries') <- countries

if (is.null(savefile)) {
  return(mask)
} else {
  ArrayToNc()
}

}


