# map script
## Fig V: Global maps of forecast biases (1) first month, (2) first season, (3) first year
## Fig VI: Global ACC maps in the (1) first season/year, (2) 2-5 years, (3) 6-10 years

#Clear workspace
rm(list = ls())

library(s2dv)
#library(lubridate)
#library(easyNCDF)
library(startR)

#///////////////////////////////////////////
#/////////    CONFIG SETTING   /////////////
#///////////////////////////////////////////

# TRUE: run the script on Nord4, serial computing
# FALSE: run the script on WS and dispatch the jobs to nord4, parallel computing
RUN_LOCAL <- FALSE
wait_Compute <- TRUE #FALSE
DEBUG <- TRUE
savefile <- TRUE
INFO <- TRUE
Write_to_log <- F
# logger
if (Write_to_log) {
  file('/esarchive/scratch/aho/cp_shiny_plots/log_map.txt', open = 'a')
  sink('/esarchive/scratch/aho/cp_shiny_plots/log_map.txt')
  print('============START MAP SCRIPT================')
}

#///////////////////////////////////////////
#///////////    VARIABLES     //////////////
#///////////////////////////////////////////

#**
exp <- 'a5b4' #'a4ge' #'a3w5'
vari <- 'tos' # 'heatc0-300m' #'tas' #'psl'
dcpp <- 'dcppA-hindcast' #'dcppB-forecast' 
mod <- 'Omon' #'Amon'
grid <- 'r360x180' #'gr'
#NOTE: era5_tas from 197901 to 202006; era5_psl start from 198001 to 201912
#sdates <- paste('s', 1979:2020)


if (exp == 'a3w5') {
  sd <- rep(paste0(1979:2020), times = 10)
  me <- rep(paste0('r', 1:10,'i4p1f1'), each = length(sd)/10)
} else if (exp == 'a4ge') {
  sd <- rep(paste0(1960:1961, '1101'), times = 3)
  me <- rep(paste0('r', 1:3,'i4p1f1'), each = length(sd)/3)
} else if (exp == 'a5b4') {
  sd <- rep(paste0(1960:1965), times = 3)
  me <- rep(paste0('r', 1:3,'i4p1f1'), each = length(sd)/3)
}
membfs_exp <- paste(sd, me , sep = '_')
sdates_fullname <- sort(unique(unlist(lapply(strsplit(membfs_exp, '_'), '[', 1)))) # "1960"
sdates <- as.numeric(sdates_fullname) #1960
membs <- sort(unique(unlist(lapply(strsplit(membfs_exp, '_'), '[', 2))))


#exp <- '__EXP__'
#vari <- '__VAR__'
#dcpp <- '__CMOR_EXP__' 
#mod <- '__TABLE__'
#grid <- '__GRID__'
#membfs_exp <- c(__SD_MEMBER_LIST__)
## membfs_exp = c(s1960_r1i1p1f1, s1961_r1i1p1f1, s1960_r2i1p1f1, ...)
#sdates <- unlist(lapply(strsplit(membfs_exp, '_'), '[', 1))
#sdates <- unique(sort(as.numeric(substr(sdates, 2, 5))))
#membs <- unique(unlist(lapply(strsplit(membfs_exp, '_'), '[', 2)))

#**
if (mod == 'Omon') {
  table <- 'ocean'
} else if (mod == 'Amon') {
  table <- 'atmos'
} else if (mod == 'SImon') {
  table == 'seaIce'
} else {
  stop("table is not defined.")
}
#savefile_path <- paste0('/esarchive/exp/ecearth/', exp, '/plots/', table, '/', vari, '/')
savefile_path <- "/esarchive/scratch/aho/cp_shiny_plots/"

#NOTE: Not used. Use all the time steps
#t1 <- 1
##t2 <- 14 # for first whole year (3:14 = JAN to DEC) 
#t2 <- 2 + (12 * 10) # fyear 1:10

lats <- c(-90, 90)
lons <- c(0, 359.9)

#///////////////////////////////////////////
#///////////    DATA LOADING     ///////////
#///////////////////////////////////////////

#--------------------------
# Step1: Load CMIP6 DCPP
#--------------------------
if (mod == 'Amon') {
  # original_files
  repos <- paste0('/esarchive/exp/ecearth/', exp, '/original_files/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/', dcpp, '/$memb$/', mod, '/$var$/', grid, '/v*/$var$_', mod,
                  '_EC-Earth3_', dcpp, '_s$sdate$-$memb$_', grid, '_$chunk$.nc')
  # [lat = 256 ; lon = 512]
} else {
  # diags
  repos <- paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/EC-Earth3/', dcpp,
                  '/$memb$/', mod, '/$var$/', grid, '/v*/$var$_', mod,
                  '_EC-Earth3_', dcpp, '_s$sdate$-$memb$_', grid, '_$chunk$.nc')
}

suppressWarnings({
data <- Start(dat = repos,
              var = vari,
              memb = membs,
              sdate = sdates_fullname, 
              #sdate_depend = 'member',
              chunk = 'all',
              chunk_depends = 'sdate',
              time = 'all', #c(t1:t2),
              time_across = 'chunk',
              merge_across_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
              #              latitude = 'all',
              latitude_reorder = Sort(),
              #              longitude = 'all',
              longitude_reorder = CircularSort(0, 360),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              #              transform = CDORemapper, transform_extra_cells = 2, transform_params = list(grid = 'r360x180', method = 'con', crop = FALSE), transform_vars = c('lat','lon'),
              return_vars = list(time = c('sdate', 'chunk'),
                                 latitude = NULL, longitude = NULL),
              silent = T, num_procs = 8,
              retrieve = FALSE)
})
# Initial month for plot titie
init_mon <- lubridate::month(attr(data, 'Variables')$common$time[1], label = TRUE)
# Get the initial month for hist and obs file selecting
first_chunk <- attr(data, 'FileSelectors')$dat1$chunk[[1]][1] # "196011-196110"
ini_month <- substr(first_chunk, 5, 6) #'11' or '05'

print('Step 1 finished. Exp is loaded.')

#--------------------------
# Step2: Load obs
#--------------------------
if (mod == 'Amon') {
  obs <- 'era5'
  path_obs <- paste0('/esarchive/recon/ecmwf/', obs, '/monthly_mean/$var$_f1h-r512x256/$var$_$file_date$.nc')
  Start_call <- 1

} else if (mod == 'Omon') {
  if (vari %in% c('tos')) {
  obs <- 'hadisst'
  path_obs <- paste0('/esarchive/obs/ukmo/hadisst_v1.1/monthly_mean/$var$/',
                     '$var$_$file_date$.nc')
  Start_call <- 1

  } else if (vari %in% c('heatc0-300m')) {
    obs <- 'en4'
    #'/esarchive/obs/ukmo/en4-v4.2.2/monitor_dcpp/heatc0-300m_en4-v4.2.2_s196011-s202112.nc'
    tmp_path <- "/esarchive/obs/ukmo/en4-v4.2.2/monitor_dcpp/"
    tmp_file <- list.files(tmp_path)
    # Keep only the desired variable
    tmp_file <- tmp_file[sapply(vari, grepl, tmp_file)]
    # Keep only the desired ini_month
    tmp_file <- tmp_file[sapply(lapply(sapply(tmp_file, strsplit, '_'), tail, 1), substr, 6, 7) == ini_month]
    if (length(tmp_file) > 1) {
      warning("There are more than 1 ", vari, " file under ", tmp_path, ". Use the first found file.")
    }

    path_obs <- paste0('/esarchive/obs/ukmo/en4-v4.2.2/monitor_dcpp/$var$_en4-v4.2.2_',
                       's*', ini_month, '-s*.nc')
    sdate_obs <- "1990-2021"
    Start_call <- 2
  }

} else {
  stop("OBS hasn't been decided yet.")
}

if (Start_call == 1) {

  #NOTE: If obs data only have one time step, then use this Start call.
  time_data <- attr(data, 'Variables')$common$time #[sdate, time]
  file_date_array <- array(format(time_data, '%Y%m'), dim = dim(time_data))
suppressWarnings({
  data_obs <- Start(dat = path_obs,
                    var = vari,
                    file_date = file_date_array,
                    split_multiselected_dims = TRUE,
                    latitude = values(list(lats[1], lats[2])),
                    longitude = values(list(lons[1], lons[2])),
                    latitude_reorder = Sort(),
                    longitude_reorder = CircularSort(0, 360),
                  #              transform = CDORemapper, transform_extra_cells = 2,
                  #              transform_params = list(grid = '/esarchive/exp/ecearth/a3w5/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/r1i1p1f1/Amon/tas/1x1/v20210910/tas_Amon_EC-Earth3_dcppA-hindcast_s19601101-r1i1p1f1_1x1_196011-196110.nc',
                  #                                      method = 'con'),
                  #              transform_vars = c('latitude', 'longitude'),
                    synonims = list(longitude = c('lon', 'longitude'),
                                    latitude = c('lat', 'latitude')),
                    return_vars = list(time = c('file_date'),
                                       latitude = NULL, longitude = NULL),
                    silent = T, num_procs = 8,
                    retrieve = FALSE)
})

} else if (Start_call == 2) {
  #NOTE: time starts in Nov or May.
  suppressWarnings({
  data_obs <- Start(dat = path_obs,
                    var = vari,
                    sdate = paste0(sdates),
                    time = 1:as.numeric(dim(attr(data, 'Variables')$common$time)['time']),
                    latitude = values(list(lats[1], lats[2])),
                    longitude = values(list(lons[1], lons[2])),
                    latitude_reorder = Sort(),
                    longitude_reorder = CircularSort(0, 360),
                    synonims = list(longitude = c('lon', 'longitude'),
                                    latitude = c('lat', 'latitude')),
                    return_vars = list(time = NULL, sdate = NULL,
                                       latitude = NULL, longitude = NULL),
                    silent = T, num_procs = 8,
                    retrieve = FALSE)
  })
}

#-----------------------------------------------------
##NOTE: This Start call may have time shift problem.
#data_obs <- Start(dat = path_obs,
#              var = vari,
#              file_date = unique(format(time_data, '%Y%m')),
#              time = values(time_data),
#              time_across = 'file_date',
#              merge_across_dims = TRUE,
#              merge_across_dims_narm = F,
#              split_multiselected_dims = TRUE,
#              latitude = values(list(lats[1], lats[2])),
#              longitude = values(list(lons[1], lons[2])),
#              latitude_reorder = Sort(),
#              longitude_reorder = CircularSort(0, 360),
##              transform = CDORemapper, transform_extra_cells = 2,
##              transform_params = list(grid = '/esarchive/exp/ecearth/a3w5/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/r1i1p1f1/Amon/tas/1x1/v20210910/tas_Amon_EC-Earth3_dcppA-hindcast_s19601101-r1i1p1f1_1x1_196011-196110.nc',
##                                      method = 'con'),
##              transform_vars = c('latitude', 'longitude'),
#              synonims = list(longitude = c('lon', 'longitude'),
#                              latitude = c('lat', 'latitude')),
#              return_vars = list(time = c('file_date'),
#                                 latitude = NULL, longitude = NULL),
#              retrieve = FALSE)

print('Step 2 finished. Obs is loaded.')

#============== CHECK ATTRIBUTES ==================
## dimensions
dim_data <- attr(data, 'Dimensions')
dim_obs <- attr(data_obs, 'Dimensions')
if (any(!names(dim_obs) %in% names(dim_data))) {
  stop("obs does not have the same dimension names as exp.")
} else if (any(dim_data[match(names(dim_obs), names(dim_data))] != dim_obs)) {
  stop("obs and exp do not have the same dimensions.")
}

## time
##NOTE: Only Start_call 1 has the correct time attributes
if (Start_call == 1) {
  time_obs <- attr(data_obs, 'Variables')$common$time 
#----------NEW_tmp----------------
# comment this check because startR has bug
#  if (!all(format(time_obs, '%Y%m')[which(!is.na(format(time_obs, '%Y%m')))] == 
#           format(time_data, '%Y%m')[which(!is.na(format(time_obs, '%Y%m')))])) {
#    stop("obs and exp do not have the same time (%Y%m).")
#  }
#----------NEW_tmp_END-----------------
} else {
  #NOTE: not used 
  time_obs <- attr(data_obs, 'Variables')$common$time  #"1 months", "2 months", etc.
  sdate_obs <- attr(data_obs, 'Variables')$common$sdate
  time_obs <- as.numeric(sapply(strsplit(time_obs, ' '), '[[', 1)) #1:132
}

## lat and lon
#lat_obs <- attr(data_obs, 'Variables')$common$latitude
#lon_obs <- attr(data_obs, 'Variables')$common$longitude
#lat_data <- attr(data, 'Variables')$common$latitude
#lon_data <- attr(data, 'Variables')$common$longitude
#if (!all.equal(lat_obs, lat_data , check.attributes = F) |
#    !all.equal(lon_obs, lon_data, check.attributes = F)) {
#  stop("obs and exp do not have the same lat/lon.")
#}
#=====================================================

browser()

#--------------------------
# Step3: Build workflow
#--------------------------
#**
#source('/esarchive/scratch/aho/cp_shiny_plots/fun_Season_diffClim_ACC.R')
source('/home/Earth/aho/aho-testtest/shiny/cp_shiny_decadal/fun_Season_diffClim_ACC.R')

step <- Step(fun = fun_Season_diffClim_ACC,
             use_libraries = c('s2dv', 'ClimProjDiags'),
             target_dims = list(data = c('sdate', 'time', 'memb'),
                                data_obs = c('sdate', 'time')),
             output_dims = list(res_bias = 'time',
                                res_acc = 'time',
                                res_acc_sig = 'time'))

wf <- AddStep(inputs = list(data, data_obs),
              step_fun = step, ini_month = as.numeric(ini_month), vari = vari)

#--------------------------
# Step4: Compute
#--------------------------
# Calculate chunks
total_data_size <- (prod(dim_data) + prod(dim_obs)) * 8 / 1024 / 1024 / 1024 #Gb
# 2GB each chunk
total_chunk <- ceiling(total_data_size / 2)
chunk_latlon <- list(latitude = total_chunk, longitude = 1)
#chunk_latlon <- list(latitude = ceiling(sqrt(total_data_size)/2), longitude = ceiling(sqrt(total_data_size)/2))
max_job_num <- ifelse(prod(unlist(chunk_latlon)) < 20, prod(unlist(chunk_latlon)), 20)

print("chunks:")
print(str(chunk_latlon))
print("max job amount:")
print(max_job_num)

if (RUN_LOCAL) {
  # local
  if (INFO) print("Compute locally.")
  output <- Compute(workflow = wf$res_bias, #diff,
                    threads_load = 2, threads_compute = 4,
                    chunks = chunk_latlon)
  dim(output$diff)
  
} else {
  # Nord3v2
  if (INFO) print("Compute on Nord3v2.")
  #-------------------user-defined---------------------
  queue_host <- 'nord4'
  temp_dir <- '/gpfs/scratch/bsc32/bsc32734/startR_hpc/'
  ecflow_suite_dir <- '/home/Earth/aho/startR_local/'
  #----------------------------------------------------
  output <- Compute(wf$res_bias,
                    chunks = chunk_latlon,
                    threads_load = 2,
                    threads_compute = 16,
                    cluster = list(
                      queue_host = queue_host,
                      queue_type = 'slurm',
                      temp_dir = temp_dir,
                      cores_per_job = 16,
                      job_wallclock = '6:00:00',
                      max_jobs = max_job_num,
                      bidirectional = FALSE,
                      polling_period = 10
                    ),
                    ecflow_suite_dir = ecflow_suite_dir,
                    wait = wait_Compute
  )
}
if (!wait_Compute) {
  saveRDS(output, paste0('~/aho-testtest/shiny/cp_shiny_decadal/mapdata_', exp,'_', vari, '_256x512_',
                         paste0(sdates[1], '-', tail(sdates, 1)), '.rds'))
  
  tmp <- readRDS(paste0('~/aho-testtest/shiny/cp_shiny_decadal/mapdata_', vari, '_256x512_', paste0(sdates[1], '-', tail(sdates, 1)), '.rds'))
  output <- Collect(tmp, wait = TRUE)
}
if (INFO) print("Compute done.")


#--------------------------
# Step5: Plotting
#--------------------------

bias <- drop(output$res_bias)
acc_data <- drop(output$res_acc)
acc_sig <- drop(output$res_acc_sig)

lon <- as.vector(attr(data, 'Variables')$common$longitude)
lat <- as.vector(attr(data, 'Variables')$common$latitude)

color_range_bias <- c(-max(abs(quantile(bias, c(0.1, 0.9), na.rm = T))),
                      max(abs(quantile(bias, c(0.1, 0.9), na.rm = T))))
#color_range_bias <- c(-5, 5)

# units for bias
var_units <- ncdf4::ncatt_get(ncdf4::nc_open(attr(data, 'ExpectedFiles')[1]), 
                              varid = vari, attname = 'units')$value
if (var_units == 0) var_units <- '' #no units

# Period of each "time" dim
period_bias <- c('Mon1', 'DJF1', 'MAM1', 'JJA1', 'SON1', 'Yr1')
period_acc <- c('DJF1', 'MAM1', 'JJA1', 'SON1', 'Yr1', 'Yr2-5', 'Yr6-10')

# bias
## Separate plots
#for (time_i in 1:dim(bias)[1]) {
#  PlotEquiMap(bias[time_i, , ], lon = lon, lat = lat, units = var_units,
#              toptitle = paste0('bias (to ', obs, ') ', exp, ', ', 
#                                vari, ', ', period_bias[time_i],
#                                ', Nm=', length(membs), ', Nsd=', length(sdates),
#                                ' (', paste(sdates, collapse = ','), ')'),
#              title_scale = 0.7,
#              filled.continents = FALSE, bar_limits = color_range_bias, 
#              fileout = paste0(savefile_path, 'map_figV_', exp, '_', vari, '_', 
#                               period_bias[time_i], '_', paste0(sdates[1], '-', tail(sdates, 1)), '.png'))
#}
## Layout
if (Start_call == 1) {
  time_obs_format <- format(time_obs, '%Y%m')[which(!is.na(format(time_obs, '%Y%m')))]
  time_obs_format_range <- paste(range(time_obs_format), collapse = '-')
} else {
  time_obs_format_range <- sdate_obs
}
main_title <- paste0('bias (to ', obs, ', ', time_obs_format_range , ') ', 
                     exp, ', ', vari, ', Init: ', init_mon,
                     ', tnm=', length(membfs_exp), ', Nsd=', length(sdates),
                     ' (first:', sdates[1], ', last:', tail(sdates, 1), ')')
PlotLayout(PlotEquiMap, c('latitude', 'longitude'), bias, lon = lon, lat = lat,
           toptitle = main_title, units = var_units,
           title_scale = 0.6, title_margin_scale = 4, bar_limits = color_range_bias,
           titles = period_bias,
           filled.continents = FALSE,
           width = 15, #height = 10,
           fileout = paste0(savefile_path, '/map_figV_', exp, '_', vari, '.png'))



# acc
## Separate plots
#for (time_i in 1:dim(acc_data)[1]) {
#  PlotEquiMap(acc_data[time_i, , ], lon = lon, lat = lat,
#              toptitle = paste0('ACC (to ', obs, ') ', exp, ', ', 
#                                vari, ', ', period_acc[time_i]),
#              filled.continents = FALSE, bar_limits = c(-1, 1), 
#              dots = acc_sig[time_i, , ],
#              fileout = paste0(savefile_path, 'map_figVI_', exp, '_', vari, '_',
#                               period_acc[time_i], '_', paste0(sdates[1], '-', tail(sdates, 1)), '.png'))
#}

## Check if any figure is empty (due to lack of forecast time)
tmp <- sapply(lapply(asplit(acc_data, 1), is.na), all)
if (any(tmp)) {
  warning("ACC figure ", paste(period_acc[which(tmp)], collapse = ', '), " not plotted due to lack of forecast time.")
  acc_data <- ClimProjDiags::Subset(acc_data, 1, list(which(!tmp)), drop = F)
  acc_sig <- ClimProjDiags::Subset(acc_sig, 1, list(which(!tmp)), drop = F)
  period_acc <- period_acc[!tmp]
}

## Layout
main_title <- paste0('ACC (to ', obs, ', ', time_obs_format_range, ') ',
                     exp, ', ', vari, ', Init: ', init_mon,
                     ', tnm=', length(membfs_exp), ', Nsd=', length(sdates),
                     ' (first:', sdates[1], ', last:', tail(sdates, 1), ')') 
special_args_list <- vector('list', length(period_acc))
dots_input <- asplit(acc_sig, 1)
special_args_list <- lapply(1:length(period_acc), function(x) {special_args_list[[x]] <- list(dots = dots_input[[x]])})

PlotLayout(PlotEquiMap, c('latitude', 'longitude'), asplit(acc_data, 1), 
           lon = lon, lat = lat,
           toptitle = main_title,
           title_scale = 0.6, title_margin_scale = 4,
           titles = period_acc,
           #width = 15, #height = 10,
           filled.continents = FALSE, bar_limits = c(-1, 1),
           special_args = special_args_list, dot_size = 0.5,
           fileout = paste0(savefile_path, 'map_figVI_', exp, '_', vari, '.png'))



if (INFO) print("==========FINISHED SUCCESSFULLY.================")

if (Write_to_log) sink()

