#Clear workspace
rm(list = ls())

#library(RColorBrewer)
library(s2dv)
library(ncdf4)
library(easyNCDF)
library(startR)
library(psych)
library(ClimProjDiags)
#**
setwd('/home/Earth/aho/aho-testtest/shiny/cp_shiny_decadal/')
source('./fun_ts_annual_mean.R')
source('./fun_ts_subset_for_clim.R')
source('./fun_nice_mfrow.R')

#///////////////////////////////////////////
#/////////    CONFIG SETTING   /////////////
#///////////////////////////////////////////

DEBUG <- TRUE
savefile <- TRUE
INFO <- TRUE
#**
Write_to_log <- F
# logger
if (Write_to_log) {
  file('/esarchive/scratch/aho/cp_shiny_plots/log_timeseries.txt', open = 'a')
  sink('/esarchive/scratch/aho/cp_shiny_plots/log_timeseries.txt')
  print('============START TIMESERIES SCRIPT================')
}

#///////////////////////////////////////////
#///////////    VARIABLES     //////////////
#///////////////////////////////////////////

#**
# Insert variables:
exp <- 'a5b4' #'a3w5'  #'a4ge' #'a4ai'
recon <- 'a3t4' #NULL #'a3t4' for Ocean and Sea ice
vari <- 'tosmean' #'tasmean' #'heatcsum0-300m'
dcpp <- 'dcppA-hindcast' #'dcppB-forecast'
mod <- 'Omon' #'Amon'
grid <- 'gn'  #'gr'
reg <- c('Global_Ocean', 'Labrador_Sea_Nudging', 'Nino3.4')
#if (is.null(region)) {
#  region <- c('Global_Ocean', 'Labrador_Sea_Nudging', 'Subpolar_Gyre_Box', 'Pacific_Ocean')
#}
full_period <- c(1960, 2020) #c(1980, 2010)

if (mod == 'Omon') {
  table <- 'ocean'
} else if (mod == 'Amon') {
  table <- 'atmos'
} else if (mod == 'SImon') {
  table == 'seaIce'
} else {
  stop("table is not defined.")
}
#savefile_path <- paste0('/esarchive/exp/ecearth/', exp, '/plots/', table, '/', vari, '/')
savefile_path <- "~/" #"/esarchive/scratch/aho/cp_shiny_plots/" 


if (exp == 'a4ai') {
  membs_exp <- c('r2i1p1f1','r3i1p1f1','r4i1p1f1','r5i1p1f1','r6i1p1f1') #, 'r7i1p1f1', 'r8i1p1f1', 'r9i1p1f1', 'r10i1p1f1', 'r11i1p1f1')
} else if (exp == 'a3w5') {
  membs_exp <- c('r1i4p1f1','r2i4p1f1','r3i4p1f1','r4i4p1f1','r5i4p1f1','r6i4p1f1','r7i4p1f1','r8i4p1f1','r9i4p1f1','r10i4p1f1') 
}

#membfs_exp <- c(__SD_MEMBER_LIST__)
# membfs_exp = c(1960_r1i1p1f1, 1961_r1i1p1f1, 1960_r2i1p1f1, ...)
if (exp == 'a4ai') {
#  sd <- rep(paste0('s', c(1980, 1981, 1985, 1986, 1990, 1991, 1996, 2000, 2001, 2005)), times = 5)
  sd <- rep(paste0(c(1980, 1981, 1985, 1986, 1990, 1991, 1996, 2000, 2001, 2005)), times = 5)
  me <- rep(paste0('r', 2:6, 'i1p1f1'), each = 10)
} else if (exp == 'a3w5') {
#  sd <- rep(paste0('s', 1960:2020), times = 10)
  sd <- rep(paste0(1960:2020), times = 10)
  me <- rep(paste0('r', 1:10,'i4p1f1'), each = length(sd)/10)
} else if (exp == 'a4ge') {
#  sd <- rep(paste0('s', 1960:1961, '1101'), times = 3)
  sd <- rep(paste0(1960:1961, '1101'), times = 3)
  me <- rep(paste0('r', 1:3,'i4p1f1'), each = length(sd)/3)
} else if (exp == 'a5b4') {
#  sd <- rep(paste0('s', 1960:1965), times = 3)
  sd <- rep(paste0(1960:1965), times = 3)
  me <-  rep(paste0('r', 1:3,'i4p1f1'), each = length(sd)/3)
}
membfs_exp <- paste(sd, me , sep = '_')
sdates_fullname <- sort(unique(unlist(lapply(strsplit(membfs_exp, '_'), '[', 1))))
sdates <- as.numeric(substr(sdates_fullname, 1, 4))
membs_exp <- sort(unique(unlist(lapply(strsplit(membfs_exp, '_'), '[', 2))))

if (any(sdates > full_period[2]) | any(sdates < full_period[1])) {
  warning("sdates: ", paste0(sdates_fullname, ','))
  warning("full_period starts: ", full_period[1])
  warning("full_period ends: ", full_period[2])
  stop("Some sdates are out of full_period. Set full_period correct to avoid consequential errors.")
}

if (INFO) {
  print(paste0("Start loading data: pred = ", exp, ", recon = ", recon, ", vari = ", vari,
               ", reg = ",  paste(reg, collapse = ',')))
}


#///////////////////////////////////////////
#///////////    DATA LOADING     ///////////
#///////////////////////////////////////////

#===== STEP1: Load CMIP6 DCPP =====

# Variable preparation
## sdate, path
###-----1. If always want the whole period:
##EC-Earth3 is changable (*) only one
#first_file_path_till_ver <- paste0(Sys.glob(paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/*/')), dcpp, '/', membs_exp[1], '/',
#                                   mod, '/', vari, '/', grid)
#
##QUESTION: two versions here in a3w5
#ver <- list.files(first_file_path_till_ver)
##if (exp == 'a4ai') {
##  files <- list.files(paste0(first_file_path_till_ver, '/', ver[1], '/'))
##} else if (exp == 'a3w5') {
##  files <- list.files(paste0(first_file_path_till_ver, '/', ver[2], '/'))  # ver[2] for a3w5
##}
#files <- list.files(paste0(first_file_path_till_ver, '/', ver[1], '/'))
#sdates_memb <- unique(sapply(lapply(lapply(files, strsplit, '_'), '[[', 1), '[', 5))
#sdates <- sapply(lapply(lapply(sdates_memb, strsplit, '-'), '[[', 1), '[', 1)  #s19601101
#if (DEBUG) {
#  if (any(substr(sdates, 1, 1) != 's') | any(sapply(sdates, nchar) != 5) |
#      any(is.na(as.integer(substr(sdates, 2, 5))))) {
#    stop("Incorrect sdate format.")
#  }
#}

##-----2. If the period is selected:
#path <- paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$memb$/', mod, '/$var$/',
#               grid, '/v*/$var$_', mod, '_EC-Earth3_dcppA-hindcast_s$sdate$1101-$memb$_', grid, '_$chunk$.nc')
#sdates <- paste(seq(yr1, yr2, by = int))
#

#if (exp %in% c('a3w5', 'a4ai') & vari %in% c('tosmean') & reg %in% c('Nino3.4', 'Global_Ocean')) {
#  data <- readRDS(paste0('/esarchive/scratch/aho/cp_shiny_plots/data_rds/data_',
#                         exp, '_', vari, '.rds'))
#  data_dates <- readRDS(paste0('/esarchive/scratch/aho/cp_shiny_plots/data_rds/data_',
#                         exp, '_', vari, '_dates.rds'))
#
#} else {
# Load data

path <- paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/*/', dcpp, '/$memb$/', mod, '/$var$/',
               grid, '/v*/$var$_', mod, '_*_', dcpp, '_$sdate$-$memb$_', grid, '_$chunk$.nc')

suppressWarnings({
data <- Start(dat = path,
              var = vari,
              memb = membs_exp,
              region = c(reg),
              sdate = sdates_fullname,
              time = 'all', #c(t1:t2),
              chunk = 'all',
              chunk_depends = 'sdate',
              time_across = 'chunk',
              merge_across_dims = TRUE,
              return_vars = list(time = c('sdate', 'chunk'), region = NULL),
              num_procs = 8,
              silent = T,
              retrieve = T)
})

data <- Subset(data, c('dat', 'var'), list(1, 1), drop = 'selected')
data_dates <- attr(data, 'Variables')$common$time
# Initial month for plot title
init_mon <- lubridate::month(data_dates[1], label = T)
# Get the initial month for hist and obs file selecting
first_chunk <- attr(data, 'FileSelectors')$dat1$chunk[[1]][1] # "196011-196110"
ini_month <- substr(first_chunk, 5, 6) #'11' or '05'
#}

if (INFO) {
  print("dim(data):")
  print(dim(data))
  print("dim(data_dates):")
  print(dim(data_dates))
}

if (DEBUG) {
  if (any(names(dim(data)) != c('memb', 'region', 'sdate', 'time'))) {
    stop("Unexpected data dimensions.")
  }
  if (any(names(dim(data_dates)) != c('sdate', 'time'))) {
    stop("Unexpected data time attribute (data_dates) dimensions.")
  }
}
if (INFO) print("==========STEP1 FINISHED.===========")

#===== STEP2: Load reconstruction =====
# Variable preparation

#if (recon %in% c('a3t4') & vari %in% c('tosmean') & reg %in% c('Nino3.4', 'Global_Ocean')) {
#data_recon <- readRDS(paste0('/esarchive/scratch/aho/cp_shiny_plots/data_rds/data_',
#                       recon, '_', vari, '.rds'))
#recon_dates <- readRDS(paste0('/esarchive/scratch/aho/cp_shiny_plots/data_rds/data_',
#                       recon, '_', vari, '_dates.rds'))
#
#} else {

if (!is.null(recon)) {
#path <- paste0('/esarchive/exp/ecearth/', recon, '/diags/CMIP/EC-Earth-Consortium/EC-Earth3-LR/piControl/$memb$/' , mod, '/$var$/',
#       grid, '/v*/$var$_', mod, '_EC-Earth3-LR_piControl_$memb$_', grid, '_$chunk$.nc')
#
#suppressWarnings({
#data_recon <- Start(dat = path,
#	    var = vari,
#	    memb = membs_recon,
#	    region = c(reg),
#	    time = 'all',
#	    chunk = 'all',
#	    #              chunk_depends = 'sdate',
#	    time_across = 'chunk',
#	    merge_across_dims = TRUE,
#	    return_vars = list(time = 'chunk', region = NULL),
#	    num_procs = 8,
#            silent = T,
#	    retrieve = T)
#})
#recon_dates <- attr(data_recon, 'Variables')$common$time
#data_recon <- Subset(data_recon, c('dat', 'var'), list(1, 1), drop = 'selected')
#data_recon <- InsertDim(data_recon, 3, 1, 'sdate')
#}

#if (INFO) {
#print("dim(data_recon):")
#print(dim(data_recon))
#print("dim(recon_dates):")
#print(dim(recon_dates))
#}
#
#if (DEBUG) {
#if (any(names(dim(data_recon)) != c('memb', 'region', 'sdate', 'time'))) {
#stop("Unexpected data_recon dimensions.")
#}
#}
#
##Hist to hind
#sdates_out <- strftime(seq(as.Date(paste0(full_period[1], "-11-01")), 
#		   as.Date(paste0(full_period[2], "-11-01")), "years"), "%Y%m")
##sdates_out <- strftime(seq(as.Date("1960-11-01"), as.Date("2020-11-01"), "years"), "%Y%m")
#data_recon_hc <- Reorder(Histo2Hindcast(data_recon, 
#				sdatesin = strftime(recon_dates[1], "%Y%m"),
#				sdatesout = sdates_out,
#				nleadtimesout = dim(data)['time'], 
#				sdate_dim = "sdate", ftime_dim = "time"),
#		 c('memb', 'region', 'sdate', 'time'))


# Find the indices of sdate and the correct ini_month name from the first member file
recon_path_1 <- paste0('/esarchive/exp/ecearth/', recon, '/monitor_dcpp/CMIP/EC-Earth-Consortium/EC-Earth3-LR/piControl/')
recon_members <- list.files(recon_path_1)
recon_path_2 <- paste0(recon_path_1, recon_members[1], '/', mod, '/', vari, '/', grid, '/')
first_file_recon <- list.files(recon_path_2, recursive = T, full.names = T)[1]

recon_sdate <- as.vector(NcToArray(first_file_recon, vars_to_read = 'sdate'))
if (any(!as.numeric(sdates) %in% recon_sdate)) {
  warning("Reconstruction data doesn't have some sdates in exp.")
}
recon_sdate_ind <- which(recon_sdate >= full_period[1] & recon_sdate <= full_period[2])

tmp <- strsplit(tail(strsplit(first_file_recon, '_')[[1]], 1), '-')[[1]][1] #s19600501 or s196005
if (nchar(tmp) == 9) {
  ini_month_recon <- paste0(ini_month, '01')
} else { # should be 7
  ini_month_recon <- ini_month
}

path_recon <- paste0(recon_path_1, '/$memb$/', mod, '/$var$/', grid, '/v*/$var$_', mod, 
                     '_EC-Earth3-LR_piControl_$memb$_', grid, '_s*', ini_month_recon, '-s*.nc')

suppressWarnings({
data_recon_hc <- Start(dat = path_recon,
               var = vari,
               memb = paste0(recon_members),
               region = c(reg),
               sdate = recon_sdate_ind,
               time = 'all',
               path_glob_permissive = 2,
               return_vars = list(time = NULL, sdate = NULL, region = NULL),
               num_procs = 8,
               silent = T,
               retrieve = T)
})
data_recon_hc <- ClimProjDiags::Subset(data_recon_hc, c('dat', 'var'), list(1, 1), drop = 'selected')
#recon_sdate <- attr(data_recon_hc, 'Variables')$common$time

# Generate the whole time series for Fig1. The first month is the initial month.
tmp <- Subset(data_recon_hc, 'time', 1:12)
data_recon <- CSTools::MergeDims(tmp, merge_dims = c('time', 'sdate'), rename_dim = 'time')
data_recon <- InsertDim(data_recon, 3, 1, name = 'sdate')

if (INFO) {
print("dim(data_recon):")
print(dim(data_recon))
}

if (DEBUG) {
if (any(names(dim(data_recon)) != c('memb', 'region', 'sdate', 'time'))) {
stop("Unexpected data_recon dimensions.")
}
}


if (INFO) {
print("dim(data_recon_hc):")
print(dim(data_recon_hc))
}

#NOTE: data_recon_hc is the subset from the first to last year of data. Even if data is discontinuous, data_recon_hc is continuous.

if (DEBUG) {
if (any(names(dim(data_recon_hc)) != c('memb', 'region', 'sdate', 'time'))) {
stop("Unexpected data_recon_hc dimensions.")
}
#if (any(dim(data)['time'] != dim(data_recon_hc)['time'])) {
#stop("The time dimension in data and data_recon_hc is not the same.")
#}
}
} # if !is.null(recon)

if (INFO) print("==========STEP2 FINISHED.===========")


#===== STEP3: Load EC-Earth historical runs =====
#QUESTION: how to store the pre-processing data? And the file name regulation to get the file automatically? Should the data be generated automatically?

# Find the indices of sdate and region from the a1tr
tmp <- paste0("/esarchive/exp/ecearth/a1tr/monitor_dcpp/CMIP/EC-Earth-Consortium/EC-Earth3/historical-ssp245/r24i1p1f1/",
		 mod, "/", vari, "/", grid, "/")
tmp_ver <- list.files(tmp)
file_hist_a1tr <- list.files(paste0(tmp, tmp_ver), full.names = T)

hist_sdate <- as.vector(NcToArray(file_hist_a1tr, vars_to_read = 'sdate'))
hist_region <- as.vector(NcToArray(file_hist_a1tr, vars_to_read = 'region'))
hist_region <- do.call(gsub, list(" ", "", hist_region))  # remove spaces

if (any(!as.numeric(sdates) %in% hist_sdate)) {
  warning("Historical data doesn't have some sdates from exp.")
}

#hist_sdate_ind <- which(hist_sdate >= 1960 & hist_sdate <= 2020)
hist_sdate_ind <- which(hist_sdate >= full_period[1] & hist_sdate <= full_period[2])
insert_lack_year <- FALSE
if (length(hist_sdate_ind) != (diff(full_period) + 1)) {
  warning("hist doesn't have the full period. It only has: ", paste(hist_sdate[hist_sdate_ind], collapse = ','))
  insert_lack_year <- TRUE
}
hist_region_ind <- which(hist_region %in% reg)
hist_region_ind <- hist_region_ind[match(hist_region[hist_region_ind], reg)]

exp_hist <- c("a247","a1st","a1sx","a1tk","a1tn","a1tt","a1wk","a1ty","a1u4","a1u8","a1to","a1tp","a1tq","a1tr","a1ts")
#exp_hist <- c("a1tr", "a1ts")
memb_hist <- c("r2","r7","r10","r12","r14","r16","r17","r18","r19","r20","r21","r22","r23","r24","r25")

# Check if there is only one file under each exp_hist
tmp_path <- paste0('/esarchive/exp/ecearth/', exp_hist, '/monitor_dcpp/CMIP/EC-Earth-Consortium/EC-Earth3/historical-ssp245/', memb_hist, 'i1p1f1/', mod, '/', vari)
#tmp_file <- list.files(tmp_path, recursive = T)
tmp_file <- lapply(tmp_path, list.files, recursive = T)
# Only keep the desired initial month
for (file_i in 1:length(tmp_file)) {
  tmp_file[[file_i]] <- tmp_file[[file_i]][sapply(lapply(sapply(tmp_file[[file_i]], strsplit, '_'), tail, 1), substr, 6, 7) == ini_month]
}
if (any(sapply(tmp_file, length) > 1)) {
  warning("Detect more than one files in some of the historial exp. The first found file in each exp will be used.")
}
if (any(sapply(tmp_file, length) == 0)) {
  stop("Not all the historical run experiments have the corresponding file. Check if any files need to be generated.")
}

path_hist <- paste0('/esarchive/exp/ecearth/', exp_hist, '/monitor_dcpp/CMIP/EC-Earth-Consortium/EC-Earth3/historical-ssp245/r*/',
	    mod, '/$var$/*/v*/$var$_', mod, '_EC-Earth3_historical-ssp245_*_*_s*', ini_month, '-s*.nc')

suppressWarnings({
cmip6_h <- Start(dat = path_hist,
	 var = vari,
	 region = hist_region_ind,
	 sdate = hist_sdate_ind,
	 time = 'all',
	 path_glob_permissive = 2,
	 return_vars = list(time = NULL, region = NULL),
         num_procs = 8,
         silent = T,
	 retrieve = T)
})

data_hist <- cmip6_h[, 1, , , ] # take off 'var'
names(dim(data_hist)) <- c("memb", "region", "sdate", "time")

if (insert_lack_year) {
  lack_yr_ind <- which(!full_period[1]:full_period[2] %in% hist_sdate)
  tmp <- array(dim = c(dim(data_hist)[1:2], sdate = (diff(full_period) + 1), dim(data_hist)[4]))
  tmp[, , -lack_yr_ind, ] <- data_hist
  data_hist <- tmp
  insert_lack_year <- FALSE
}

#if (dim(data_hist)['sdate'] != 61) { # not 1960-2020
#  arr_tmp <- array(dim = c(dim(data_hist)[c('memb', 'region')], sdate = 61, dim(data_hist)['time']))
#  arr_tmp[,  , hist_sdate[hist_sdate_ind] - 1959, ] <- data_hist
#  data_hist <- arr_tmp
#  rm(arr_tmp)
#}

#for (i_reg in reg) {
#  nc_file <- paste0('/esarchive/scratch/Earth/rfernand/monitor_dcpp/hist_', vari, '_', i_reg, '_s1960-s2022.nc')
#  nc_data <- nc_open(nc_file)
#  tmp <- ncvar_get(nc_data, vari)
#  nc_close(nc_data)
#  if (!exists("data_hist")) {
#    data_hist <- array(dim = c(dim(tmp), length(reg)))
#  }
#  data_hist[, , , which(reg == i_reg)] <- tmp
#}
#data_hist <- Reorder(data_hist, c(1, 4, 2, 3))
#names(dim(data_hist)) <- c("memb", "region", "sdate", "time")


if (INFO) {
print("dim(data_hist):")
print(dim(data_hist))
}

if (INFO) print("==========STEP3 FINISHED.===========")

#===== STEP4: Load OBS =====
#QUESTION: same as above
#if (tolower(obs) == 'hadisst') {
if (vari == 'tosmean') {
  obs <- 'hadisst'
  tmp_path <- '/esarchive/obs/ukmo/hadisst_v1.1/monitor_dcpp/'
#  obs_file <- paste0('/esarchive/obs/ukmo/hadisst_v1.1/monitor_dcpp/', vari, 
#                     '_hadisst_v1.1_s1960-s2020.nc')

#} else if (tolower(obs) == 'en4') {
} else if (vari == "heatcsum0-300m") {
  obs <- 'en4'
  tmp_path <- '/esarchive/obs/ukmo/en4-v4.2.2/monitor_dcpp/'
#  obs_file <- paste0('/esarchive/obs/ukmo/en4-v4.2.2/monitor_dcpp/', vari,
#                     '_en4-v4.2.2_s1990-s2021.nc')

#} else if (tolower(obs) == 'era5') {
} else if (vari == 'tasmean') {
  obs <- 'era5'
  tmp_path <- '/esarchive/recon/ecmwf/era5/monitor_dcpp/'
#  obs_file <- paste0('/esarchive/recon/ecmwf/era5/monitor_dcpp/', vari,
#                     '_era5_s1979-s2020.nc')

} else {
  stop("Obs hasn't been decided.")
}

obs_file <- list.files(tmp_path, full.names = T)
# Keep only the desired variable
obs_file <- obs_file[sapply(vari, grepl, obs_file)]
# Keep only the desired initial month
obs_file <- obs_file[sapply(lapply(sapply(obs_file, strsplit, '_'), tail, 1), substr, 6, 7) == ini_month]

if (length(obs_file) > 1) {
  obs_file <- obs_file[1]
  warning("There are more than 1 ", vari, " files under ", tmp_path, ". Use the first found file, ",
          obs_file, ".")
}

obs_sdate <- as.vector(NcToArray(obs_file, vars_to_read = 'sdate'))
obs_region <- as.vector(NcToArray(obs_file, vars_to_read = 'region'))
obs_region <- do.call(gsub, list(" ", "", obs_region))  # remove spaces

#obs_sdate_ind <- which(obs_sdate >= 1960 & obs_sdate <= 2020)
obs_sdate_ind <- which(obs_sdate >= full_period[1] & obs_sdate <= full_period[2])
if (length(obs_sdate_ind) != (diff(full_period) + 1)) {
  warning("obs doesn't have the full period. It only has: ", paste(obs_sdate[obs_sdate_ind], collapse = ','))
  insert_lack_year <- TRUE
}

obs_region_ind <- which(obs_region %in% reg)
obs_region_ind <- obs_region_ind[match(obs_region[obs_region_ind], reg)]
if (length(obs_region_ind) != length(reg)) {
  warning("Some regions are missing in obs.")
}
data_obs <- NcToArray(obs_file, vars_to_read = vari, 
	      dim_indices = list(sdate = obs_sdate_ind,
				 region = obs_region_ind))
data_obs <- Reorder(data_obs, c("var", "region", "sdate", "time"))
names(dim(data_obs))[1] <- "memb"

if (insert_lack_year) {
  lack_yr_ind <- which(!full_period[1]:full_period[2] %in% obs_sdate)
  tmp <- array(dim = c(dim(data_obs)[1:2], sdate = (diff(full_period) + 1), dim(data_obs)[4]))
  tmp[, , -lack_yr_ind, ] <- data_obs
  data_obs <- tmp
  insert_lack_year <- FALSE
}

# If regions are in separate files
#for (i_reg in reg) {
#  nc_file <- paste0('/esarchive/scratch/Earth/rfernand/monitor_dcpp/', obs, '_', vari, '_', i_reg, '_s1960-s2020.nc')
#  nc_data <- nc_open(nc_file)
#  tmp <- InsertDim(ncvar_get(nc_data, "tos"), 1, 1, 'memb') - 273.15
#  nc_close(nc_data)
#  if (!exists("data_obs")) {
#    data_obs <- array(dim = c(dim(tmp), length(reg)))
#  }
#  data_obs[, , , which(reg == i_reg)] <- tmp
#}
#data_obs <- Reorder(data_obs, c(1, 4, 2, 3))
#names(dim(data_obs)) <- c("memb", "region", "sdate", "time")


if (INFO) {
print("dim(data_obs)")
print(dim(data_obs))
}

if (INFO) print("==========STEP4 FINISHED.===========")

if (is.null(recon)) {
  print("Use obs as recon for Fig1.")
  tmp <- Subset(data_obs, 'time', 1:12)
  data_recon <- CSTools::MergeDims(tmp, merge_dims = c('time', 'sdate'), rename_dim = 'time')
  data_recon <- InsertDim(data_recon, 3, 1, name = 'sdate')

#  # Tune data: from 1960/01 to 2020/12 (now it is 1960/11 to 2021/10)
#  tmp <- array(dim = c(dim(data_recon)[1:3], time = as.numeric(dim(data_recon)['time']) + 10))
#  tmp[, , , 11:dim(tmp)['time']] <- data_recon
#  # Remove the last 10 months; the last month is 202012
#  data_recon <- ClimProjDiags::Subset(tmp, 'time', list(1:dim(data_recon)['time']), drop = F)
}

#///////////////////////////////////////////
#//////////    DIAGNOSTICS     /////////////
#///////////////////////////////////////////

if (length(unique(dim(data)['region'], dim(data_recon)['region'], dim(data_hist)['region'], dim(data_obs)['region'])) != 1) {
  stop("Not all the datasets have the same number of region. Check if all the datasets get the same regions.")
}

## fig II
reg_data_ss_anom_ens <- reg_recon_ss_anom_ens <- reg_hist_ss_anom_ens <-
  reg_obs_ss_anom_ens <- reg_hist_ss_full_anom_ens <- reg_obs_ss_full_anom_ens <-
  reg_recon_ss_full_anom_ens <- vector('list', length = length(reg))

## fig III
reg_clim_data <- reg_clim_recon <- reg_clim_hist <- reg_clim_hist_full <-
  reg_clim_recon_full <- vector('list', length = length(reg))

## fig IV
reg_acc_data <- reg_acc_hist <- reg_acc_hist_full <- reg_acc_data_recon <-
  reg_acc_hist_recon <- reg_acc_hist_recon_full <- vector('list', length = length(reg))

reg_acc_data_sig <- reg_acc_hist_sig <- reg_acc_hist_full_sig <- reg_acc_data_recon_sig <-
  reg_acc_hist_recon_sig <- reg_acc_hist_recon_full_sig <- vector('list', length = length(reg))

reg_sig_corr <- reg_sig_corr_recon <- vector('list', length = length(reg))

# t2 is the xth year since full_period[1].
# E.g., full_period[1] = 1960: 1960 is 1st yr, 1961 is 2nd yr, etc.
#t2 <- as.numeric(substr(data_dates[, 1], 1, 4)) - as.numeric(substr(recon_dates[1], 1, 4)) + 1
t2 <- as.numeric(substr(data_dates[, 1], 1, 4)) - full_period[1] + 1

for (i_reg in reg) {
  if (INFO) {
    print(paste0("Start region: ", i_reg))
  }
  # Subset region
  i_data <- Subset(data, 'region', which(reg == i_reg), drop = 'selected')
  if (!is.null(recon)) {
  i_data_recon <- Subset(data_recon, 'region', which(reg == i_reg), drop = 'selected')
  i_data_recon_hc <- Subset(data_recon_hc, 'region', which(reg == i_reg), drop = 'selected')
  }
  i_data_hist <- Subset(data_hist, 'region', which(reg == i_reg), drop = 'selected')
  i_data_obs <- Subset(data_obs, 'region', which(reg == i_reg), drop = 'selected')
  
  #===== STEP5: Compute annual means ===== (only global mean)
  #Question: Which variables need to compute annual mean? Which use monthly data directly?
  
  if (!i_reg %in% c('Nino3.4')) {
    #exp
    data_yr <- fun_ts_annual_mean(i_data)
    #recon
    if (!is.null(recon)) {
    data_recon_yr <- fun_ts_annual_mean(i_data_recon_hc)
    }
    #Hist
    data_hist_yr <- fun_ts_annual_mean(i_data_hist)
    #obs
    data_obs_yr <- fun_ts_annual_mean(i_data_obs)
  }
  
  if (INFO) print("==========STEP5 FINISHED.===========")
  
  
  #===== STEP6: Compute Clim and Ano =====
  
  #Extract the same number of startdates and member
  
  #QUESTION: why 60? Can it be 61 (the shortest sdate)?
  ##Not used
  #sds <- 60 # min(sapply(list(dim(data_yr), dim(data_recon_yr), dim(data_hist_yr), dim(data_obs_yr)), '[', 'sdate')) # --> it doesn't work for discontinuous data (a4ai)
  
  # If Nino3.4, the first 30 months; if Global_Means, all the years
  time_len <- ifelse(i_reg %in% c('Nino3.4'), 30, dim(data_yr)['time'])
  if (i_reg %in% c('Nino3.4')) {
    data_tmp <- i_data
    if (!is.null(recon)) recon_tmp <- i_data_recon_hc
    hist_tmp <- i_data_hist
    obs_tmp <- i_data_obs
  } else {
    data_tmp <- data_yr
    if (!is.null(recon)) recon_tmp <- data_recon_yr
    hist_tmp <- data_hist_yr
    obs_tmp <- data_obs_yr
  }
  
  # Subset to the same time and sdate; hist has the same member as data
  memb_len <- min(dim(hist_tmp)['memb'], dim(data_tmp)['memb'])
  #exp
  data_ss <- fun_ts_subset_for_clim(data_tmp, 'data', time_len, memb_len = NULL, sdate_ind = NULL)
  #recon
    if (!is.null(recon)) recon_ss <- fun_ts_subset_for_clim(recon_tmp, 'recon', time_len, memb_len = NULL, sdate_ind = t2)
  #hist
  hist_ss <- fun_ts_subset_for_clim(hist_tmp, 'hist', time_len, memb_len = memb_len, sdate_ind = t2)
  #obs
  obs_ss <- fun_ts_subset_for_clim(obs_tmp, 'obs', time_len, memb_len = NULL, sdate_ind = t2)
  
  # hist needs to have a full sdate and member line, and obs and recon need to have 
  # a full sdate to compare with full hist.
  hist_ss_full <- fun_ts_subset_for_clim(hist_tmp, 'hist', time_len, memb_len = dim(hist_tmp)['memb'], sdate_ind = 1:(dim(hist_tmp)['sdate']))
  obs_ss_full <- fun_ts_subset_for_clim(obs_tmp, 'obs', time_len, memb_len = NULL, sdate_ind = 1:dim(obs_tmp)['sdate'])
    if (!is.null(recon)) recon_ss_full <- fun_ts_subset_for_clim(recon_tmp, 'recon', time_len, memb_len = NULL, sdate_ind = 1:dim(recon_tmp)['sdate'])

  if (INFO) {
    print("dim(data_ss):")
    print(dim(data_ss))
    print("dim(recon_ss):")
    if (!is.null(recon)) print(dim(recon_ss))
    print("dim(hist_ss):")
    print(dim(hist_ss))
    print("dim(obs_ss):")
    print(dim(obs_ss))
    
    print("dim(hist_ss_full):")
    print(dim(hist_ss_full))
    print("dim(obs_ss_full):")
    print(dim(obs_ss_full))
    print("dim(recon_ss_full):")
    if (!is.null(recon)) print(dim(recon_ss_full))
  }
  
  if (DEBUG) {
    tmp_dimnames <- c('dat', 'memb', 'sdate', 'time')
    if (!is.null(recon)) {
      if (any(names(dim(data_ss)) != tmp_dimnames) | 
          any(names(dim(recon_ss)) != tmp_dimnames) |
          any(names(dim(hist_ss)) != tmp_dimnames) |
          any(names(dim(obs_ss)) != tmp_dimnames)) {
        stop("Unexpected xxx_ss dimensions. They're used as Clim input.")
      }

      if (!identical(
           dim(data_ss)[c('sdate', 'time')], 
           dim(recon_ss)[c('sdate', 'time')],
           dim(hist_ss)[c('sdate', 'time')], 
           dim(obs_ss)[c('sdate', 'time')])) {
        stop("The data inputs for Clim should have the same sdate and time dimensions.")
      }
      # full hist and obs
      if (!identical(
            dim(hist_ss_full)[c('sdate', 'time')],
            dim(obs_ss_full)[c('sdate', 'time')],
            dim(recon_ss_full)[c('sdate', 'time')])) {
        stop("The data inputs for Clim should have the same sdate and time dimensions. hist_full and obs_full don't share the same dimensions.")
      }

    } else {  #is.null(recon)
      if (any(names(dim(data_ss)) != tmp_dimnames) |
          any(names(dim(hist_ss)) != tmp_dimnames) |
          any(names(dim(obs_ss)) != tmp_dimnames)) {
        stop("Unexpected xxx_ss dimensions. They're used as Clim input.")
      }
      if (!identical(
           dim(data_ss)[c('sdate', 'time')],
           dim(hist_ss)[c('sdate', 'time')],
           dim(obs_ss)[c('sdate', 'time')])) {
        stop("The data inputs for Clim should have the same sdate and time dimensions.")
      }
      # full hist and obs
      if (!identical(
           dim(hist_ss_full)[c('sdate', 'time')],
           dim(obs_ss_full)[c('sdate', 'time')])) {
        stop("The data inputs for Clim should have the same sdate and time dimensions. hist_full and obs_full don't share the same dimensions.")
      }
    }
  }

  # Calculate Clim
  arg_list <- list(time_dim = "sdate", dat_dim = c("dat", "memb"), method = "clim", 
                   ftime_dim = "time", memb_dim = "memb", memb = FALSE, na.rm = TRUE)
  clim_data <- do.call(Clim, c(list(data_ss, obs_ss), arg_list))
  reg_clim_data[[which(reg == i_reg)]] <- clim_data
  if (!is.null(recon)) {
    clim_recon <- do.call(Clim, c(list(recon_ss, obs_ss), arg_list))
    reg_clim_recon[[which(reg == i_reg)]] <- clim_recon
  }
  clim_hist <- do.call(Clim, c(list(hist_ss, obs_ss), arg_list))
  reg_clim_hist[[which(reg == i_reg)]] <- clim_hist
  
  # Full period (all sdates)
  clim_hist_full <- do.call(Clim, c(list(hist_ss_full, obs_ss_full), arg_list))
  reg_clim_hist_full[[which(reg == i_reg)]] <- clim_hist_full
  if (!is.null(recon)) {
    clim_recon_full <- do.call(Clim, c(list(recon_ss_full, obs_ss_full), arg_list))
    reg_clim_recon_full[[which(reg == i_reg)]] <- clim_recon_full
  }

  if (INFO) {
    print("====Clim output===========")
    print("dim(clim_data$clim_exp):")
    print(dim(clim_data$clim_exp))
    print("dim(clim_recon$clim_exp):")
    if (!is.null(recon)) print(dim(clim_recon$clim_exp))
    print("dim(clim_data$clim_obs):")
    print(dim(clim_data$clim_obs))
    print("dim(clim_hist$clim_exp):")
    print(dim(clim_hist$clim_exp))
    print("dim(clim_hist_full):")
    print(dim(clim_hist_full$clim_exp))
    print("dim(clim_recon_full):")
    if (!is.null(recon)) print(dim(clim_recon_full$clim_exp))
  }
  
  # Calculate Anomalies & Ensemble mean
  data_ss_anom <- Ano(data_ss, clim_data$clim_exp)
  data_ss_anom_ens <- MeanDims(data_ss_anom, 'memb')
  reg_data_ss_anom_ens[[which(reg == i_reg)]] <- data_ss_anom_ens

  if (!is.null(recon)) {  
  recon_ss_anom <- Ano(recon_ss, clim_recon$clim_exp)
  recon_ss_anom_ens <- MeanDims(recon_ss_anom, 'memb')
  reg_recon_ss_anom_ens[[which(reg == i_reg)]] <- recon_ss_anom_ens
  }
  
  obs_ss_anom <- Ano(obs_ss, clim_data$clim_obs)
  obs_ss_anom_ens <- MeanDims(obs_ss_anom, 'memb')
  reg_obs_ss_anom_ens[[which(reg == i_reg)]] <- obs_ss_anom_ens
  
  hist_ss_anom <- Ano(hist_ss, clim_hist$clim_exp)
  hist_ss_anom_ens <- MeanDims(hist_ss_anom, 'memb')
  reg_hist_ss_anom_ens[[which(reg == i_reg)]] <- hist_ss_anom_ens
  
  hist_ss_full_anom <- Ano(hist_ss_full, clim_hist_full$clim_exp)
  hist_ss_full_anom_ens <- MeanDims(hist_ss_full_anom, 'memb')
  reg_hist_ss_full_anom_ens[[which(reg == i_reg)]] <- hist_ss_full_anom_ens
  
  obs_ss_full_anom <- Ano(obs_ss_full, clim_hist_full$clim_obs)
  obs_ss_full_anom_ens <- MeanDims(obs_ss_full_anom, 'memb')
  reg_obs_ss_full_anom_ens[[which(reg == i_reg)]] <- obs_ss_full_anom_ens

  if (!is.null(recon)) {  
  recon_ss_full_anom <- Ano(recon_ss_full, clim_recon_full$clim_exp)
  recon_ss_full_anom_ens <- MeanDims(recon_ss_full_anom, 'memb')
  reg_recon_ss_full_anom_ens[[which(reg == i_reg)]] <- recon_ss_full_anom_ens
  }
 
  if (INFO) {
    print("====Ano+ens_mean output===========")
    print("dim(data_ss_anom_ens):")
    print(dim(data_ss_anom_ens))
    print("dim(recon_ss_anom_ens):")
  if (!is.null(recon)) print(dim(recon_ss_anom_ens))
    print("dim(obs_ss_anom_ens):")
    print(dim(obs_ss_anom_ens))
    print("dim(hist_ss_anom_ens):")
    print(dim(hist_ss_anom_ens))
    print("dim(hist_ss_full_anom_ens):")
    print(dim(hist_ss_full_anom_ens))
    print("dim(obs_ss_full_anom_ens):")
    print(dim(obs_ss_full_anom_ens))
    print("dim(recon_ss_full_anom_ens):")
  if (!is.null(recon)) print(dim(recon_ss_full_anom_ens))
  }
  
  if (DEBUG) {
    tmp_dimnames <- c('dat', 'sdate', 'time')
    if (!is.null(recon)) {  
      if (any(names(dim(data_ss_anom_ens)) != tmp_dimnames) |
          any(names(dim(recon_ss_anom_ens)) != tmp_dimnames) |
          any(names(dim(hist_ss_anom_ens)) != tmp_dimnames) |
          any(names(dim(obs_ss_anom_ens)) != tmp_dimnames)) {
        stop("Unexpected xxx_ss_anom_ens dimensions.")
      }
      if (!identical(
           dim(data_ss_anom_ens)[c('sdate', 'time')],
           dim(recon_ss_anom_ens)[c('sdate', 'time')],
           dim(hist_ss_anom_ens)[c('sdate', 'time')],
           dim(obs_ss_anom_ens)[c('sdate', 'time')])) {
        stop("The data inputs for Clim should have the same sdate and time dimensions.")
      }
      if (!identical(
           dim(hist_ss_full_anom_ens)[c('sdate', 'time')],
           dim(obs_ss_full_anom_ens)[c('sdate', 'time')],
           dim(recon_ss_full_anom_ens)[c('sdate', 'time')])) {
        stop("The data inputs for Clim should have the same sdate and time dimensions. Not same for full hist and full obs.")
      }

    } else {  #is.null(recon)
      if (any(names(dim(data_ss_anom_ens)) != tmp_dimnames) |
          any(names(dim(hist_ss_anom_ens)) != tmp_dimnames) |
          any(names(dim(obs_ss_anom_ens)) != tmp_dimnames)) {
        stop("Unexpected xxx_ss_anom_ens dimensions.")
      }
      if (!identical(
           dim(data_ss_anom_ens)[c('sdate', 'time')],
           dim(hist_ss_anom_ens)[c('sdate', 'time')],
           dim(obs_ss_anom_ens)[c('sdate', 'time')])) {
        stop("The data inputs for Clim should have the same sdate and time dimensions.")
      }
      if (!identical(
           dim(hist_ss_full_anom_ens)[c('sdate', 'time')],
           dim(obs_ss_full_anom_ens)[c('sdate', 'time')])) {
        stop("The data inputs for Clim should have the same sdate and time dimensions. Not same for full hist and full obs.")
      }
    }
  }
  
  if (INFO) print("==========STEP6 FINISHED.===========")
  
  #===== STEP7: Compute Skill (ACC) =====
  corr_arg_list <- list(time_dim = 'sdate', dat_dim = 'dat', method = 'pearson', pval = TRUE, conf = F)
  # figIV
  acc_data <- do.call(Corr, c(list(data_ss_anom_ens, obs_ss_anom_ens), corr_arg_list))
  reg_acc_data[[which(reg == i_reg)]] <- acc_data
  acc_hist <- do.call(Corr, c(list(hist_ss_anom_ens, obs_ss_anom_ens), corr_arg_list))
  reg_acc_hist[[which(reg == i_reg)]] <- acc_hist
  acc_hist_full <- do.call(Corr, c(list(hist_ss_full_anom_ens, obs_ss_full_anom_ens), corr_arg_list))
  reg_acc_hist_full[[which(reg == i_reg)]] <- acc_hist_full
 
  if (!is.null(recon)) { 
  # for figIVb
  acc_data_recon <- do.call(Corr, c(list(data_ss_anom_ens, recon_ss_anom_ens), corr_arg_list))
  reg_acc_data_recon[[which(reg == i_reg)]] <- acc_data_recon
  acc_hist_recon <- do.call(Corr, c(list(hist_ss_anom_ens, recon_ss_anom_ens), corr_arg_list))
  reg_acc_hist_recon[[which(reg == i_reg)]] <- acc_hist_recon
  acc_hist_recon_full <- do.call(Corr, c(list(hist_ss_full_anom_ens, recon_ss_full_anom_ens), corr_arg_list))
  reg_acc_hist_recon_full[[which(reg == i_reg)]] <- acc_hist_recon_full
}

  if (DEBUG) {
    if (!is.null(recon)) {  
      if (!identical(
        dim(acc_data$corr)['time'],
        dim(acc_hist$corr)['time'],
        dim(acc_data_recon$corr)['time'],
        dim(acc_hist_recon$corr)['time'])) {
        stop("The ACC outputs should have the same time dimensions.")
      }
    } else {
      if (!identical(
        dim(acc_data$corr)['time'],
        dim(acc_hist$corr)['time'])) {
        stop("The ACC outputs should have the same time dimensions.")
      }
    }
  }

  # ACC significance: Whether the correlation is significant
  fun_acc_sig <- function(pval, alpha = 0.05) {
    # pval: [time]
    acc_sig <- as.integer(pval < alpha)
    acc_sig[acc_sig == 0] <- NaN
    return(acc_sig)
  }
  
  reg_acc_data_sig[[which(reg == i_reg)]] <- fun_acc_sig(acc_data$p.val[1, 1, ])
  #  acc_data_sig <- as.integer(acc_data$p.val[1, , ] < 0.05)
  #  acc_data_sig[acc_data_sig == 0] <- NaN
  #  reg_acc_data_sig[[which(reg == i_reg)]] <- acc_data_sig
  reg_acc_hist_sig[[which(reg == i_reg)]] <- fun_acc_sig(acc_hist$p.val[1, 1, ])
  reg_acc_hist_full_sig[[which(reg == i_reg)]] <- fun_acc_sig(acc_hist_full$p.val[1, 1, ])

  if (!is.null(recon)) {
  reg_acc_data_recon_sig[[which(reg == i_reg)]] <- fun_acc_sig(acc_data_recon$p.val[1, 1, ])
  reg_acc_hist_recon_sig[[which(reg == i_reg)]] <- fun_acc_sig(acc_hist_recon$p.val[1, 1, ])
  reg_acc_hist_recon_full_sig[[which(reg == i_reg)]] <- fun_acc_sig(acc_hist_recon_full$p.val[1, 1, ])
  }

  # ACC difference significance: Whether the difference in correlation between data and hist is significant
  
  ## 1) ACC of the PRED and HIST (to take auto-correlation into account.)
  ohc_acc_mod <- do.call(Corr, c(list(data_ss_anom_ens, hist_ss_anom_ens), corr_arg_list))
  ## 2) Effective sample size
  eff_sample <- Eno(obs_ss_anom, time_dim = 'sdate')[1, , ]
  ## 3) Compute significance with paired.r function
  sig_corr <- psych::paired.r(acc_data$corr[1, , ], acc_hist$corr[1, , ],
                              ohc_acc_mod$corr[1, , ], round(eff_sample),
                              twotailed = TRUE)$p < 0.05
  #change T/F to 1/0
  sig_corr <- sig_corr * 1
  sig_corr[sig_corr == 0] <- NaN
  reg_sig_corr[[which(reg == i_reg)]] <- sig_corr
 
  if (!is.null(recon)) { 
  sig_corr_recon <- psych::paired.r(acc_data_recon$corr[1, , ], acc_hist_recon$corr[1, , ],
                                    ohc_acc_mod$corr[1, , ], round(eff_sample),
                                    twotailed = TRUE)$p < 0.05
  #change T/F to 1/0
  sig_corr_recon <- sig_corr_recon * 1
  sig_corr_recon[sig_corr_recon == 0] <- NaN
  reg_sig_corr_recon[[which(reg == i_reg)]] <- sig_corr_recon
  }
  
  if (INFO) print("==========STEP7 FINISHED.===========")
  
}
#browser()

#////////////////////////////////////////////
#//////////////    PLOT    //////////////////
#////////////////////////////////////////////

vari_units <- attr(data, 'Variables')$common[[vari]]$units
info_tnm <- paste0('tnm=', length(membfs_exp), '; Nsd=', length(t2), '; Nm=', dim(i_data)['memb'])

#===== Plotting: Timeseries based on monthly means (type 1a) =====

pcexlab <- 1.3
pcexmain <- 1.6
pcexax <- 1.3
cex.legend <- 1.2
cex.title <- 1.4
cex.sub_title <- 0.8

nf <- 6 # first 6 ftime

for (plot_i in 1:2) {

if (savefile) {
  png(paste0(savefile_path, 'timeseries_figIa', plot_i, '_', exp, '_', vari, '.png'),
      width = 480 * 1.8, height = 480 * 0.4 * length(reg))
} else {
  dev.new(width = 9, height = 2.5 * length(reg))
}
par(mfrow = c(length(reg), 1), cex = 0.65) #, mar = c(5.1, 4.1, 6.1, 2.1))

#  brks_def <- c(seq(1, dim(i_data)['sdate'], by = 4))
#  colors <- readRDS('/esarchive/scratch/Earth/rfernand/dcpp/colorbar_21.RDS')
#  col_fun <- colorRampPalette(colors[1:20])
#  jBrewColorsano <- c(col_fun(length(brks_def) - 1))

for (i_reg in 1:length(reg)) {
  i_data <- Subset(data, 'region', i_reg, drop = 'selected')
  i_data_recon <- Subset(data_recon, 'region', i_reg, drop = 'selected')
#  if (!is.null(recon)) {
#    # Subset to full_period
#    recon_sdate_ind <- which(recon_dates >= as.POSIXct(paste0(full_period[1], "-01-01"), tz = 'UTC') & 
#                             recon_dates <= as.POSIXct(paste0(full_period[2], "-12-31"), tz = 'UTC'))
#    i_data_recon <- Subset(i_data_recon, 'time', recon_sdate_ind)
#  } else {
#    i_data_recon <- i_data_recon
#  }

  max_range <- range(range(i_data[, , 1:nf], na.rm = T), 
                     range(i_data_recon, na.rm = T),
                     finite = T)
  enlarge_ratio <- 0.00
  ylim <- c(max_range[1] - diff(max_range) * enlarge_ratio, max_range[2] + diff(max_range) * enlarge_ratio)
 
  x_num <- (diff(full_period) + 1) * 12
  if (plot_i == 1) {
#    xlim_i <- c(1, 350)
    xlim_i <- c(1, round(x_num / 2))
  } else {
#    xlim_i <- c(351, 732)
    xlim_i <- c(round(x_num / 2) + 1, x_num)
  }
  # create the frame and draw the first member
  plot(c(1:x_num), i_data_recon[1, , ], col = "grey", type = "l", lwd = 0.5, 
       ylim = ylim, xlim = xlim_i, cex.lab = pcexlab, #cex.main = pcexmain, 
       cex.axis = pcexax, xlab = "Time", ylab = paste0("T (", vari_units, ")"), lty = 1, xaxt = "n")
  title(main = reg[i_reg], adj = 0, cex.main = pcexmain, line = 0.6)
#  axis(1, at = seq(1, x_num, by = 12), labels = paste0(seq(as.numeric(substr(recon_dates[1], 1, 4)), as.numeric(substr(recon_dates[length(recon_dates)], 1, 4)))))
  axis(1, at = seq(1, x_num, by = 12), labels = paste0(full_period[1]:full_period[2]))

  # draw the rest of member
  if (dim(i_data_recon)['memb'] > 1) {
    for (i in seq(2, dim(i_data_recon)['memb'])) {
      lines(c(1:x_num), i_data_recon[i, , ], col = "grey", type = "l", lwd = 0.5)
    }
  }
  
  # draw exp, memb by memb, sdate by sdate
  for (i in seq(1, dim(i_data)['memb'])) {
    for (t in seq(1, length(t2))) {
      # the chosen ftime (1:nf) of each year. The first month is the ini month (11 or 05) already.
      lines(c(1:nf) + (12 * (t2[t] - 1)), i_data[i, t, 1:nf], col = 'blue', type = "l", lwd = 0.5) 
    }
  }
  # add legend
  if (!is.null(recon)) {
    legend(xlim_i[1], ylim[2], legend = c(exp, recon), col = c("blue", 'grey'), lty = 1, bty = "n", cex = cex.legend)
  } else {
    legend(xlim_i[1], ylim[2], legend = c(exp, obs), col = c("blue", 'grey'), lty = 1, bty = "n", cex = cex.legend)
  }

} # region for loop

# Add common main title
if (plot_i == 1) {
mtext(paste0('Time series monthly mean (1st half), ', vari, ', ', exp, ', Init: ', init_mon),
      side = 3, line = -2, outer = TRUE, cex = cex.title)
} else {
mtext(paste0('Time series monthly mean (2st half), ', vari, ', ', exp, ', Init: ', init_mon),
      side = 3, line = -2, outer = TRUE, cex = cex.title)
}
mtext(info_tnm, side = 3, line = -3.5, outer = TRUE, cex = cex.sub_title)

if (savefile) dev.off()

} # plot_i for loop  


if (INFO) print("==========PLOTTING TYPE I FINISHED.===========")


#===== Plotting: type II =====
if (savefile) {
  png(paste0(savefile_path, 'timeseries_figII_', exp, '_', vari, '.png'),
      width = 480 * 1.8, height = 480 * 0.6 * length(reg))
} else {
  dev.new(width = 9, height = 5 * length(reg))
}
par(mfrow = c(length(reg), 1), cex = 0.65)

pcexlab <- 1.3
pcexmain <- 1.6
pcexax <- 1.3
cex.legend <- 1.2
cex.title <- 1.4
cex.sub_title <- 0.8

for (i_reg in 1:length(reg)) {

#TODO: Is it correct?
  if (!reg[i_reg] %in% c('Nino3.4')) {
#    yrs <- seq(1, t2[length(t2)] - t2[1] + 1, by = 1) + as.numeric(substr(sdates[1], 2, 5))  #start from 1961
#    yrs <- 1961:2021
    yrs <- full_period[1]:full_period[2]
  } else {
#    yrs <- seq(1, t2[length(t2)] - t2[1] + 1, by = 1) + as.numeric(substr(sdates[1], 2, 5)) - 1 #start from 1960
#    yrs <- 1960:2020
    yrs <- full_period[1]:full_period[2]
  }
  
  # leadtime
  lt <- 1
  if (!is.null(recon)) {
    max_range <- range(range(reg_data_ss_anom_ens[[i_reg]][1, , lt], na.rm = T),
                       range(reg_recon_ss_anom_ens[[i_reg]][1, , lt], na.rm = T),
                       range(reg_hist_ss_anom_ens[[i_reg]][1, , lt], na.rm = T),
                       range(reg_obs_ss_anom_ens[[i_reg]][1, , lt], na.rm = T),
                       range(reg_hist_ss_full_anom_ens[[i_reg]][1, , lt], na.rm = T), 
                       na.rm = T, finite = T)
  } else {
    max_range <- range(range(reg_data_ss_anom_ens[[i_reg]][1, , lt], na.rm = T),
                       range(reg_hist_ss_anom_ens[[i_reg]][1, , lt], na.rm = T),
                       range(reg_obs_ss_anom_ens[[i_reg]][1, , lt], na.rm = T),
                       range(reg_hist_ss_full_anom_ens[[i_reg]][1, , lt], na.rm = T),
                       na.rm = T, finite = T)
  }
  enlarge_ratio <- 0.0
  ylim <- c(max_range[1] - diff(max_range) * enlarge_ratio, max_range[2] + diff(max_range) * enlarge_ratio)
  
  type_sdate <- ifelse(all(diff(t2) == 1), 'l', 'p')
  
  if (all(diff(t2) == 1)) { #continuous sdate
    plot(yrs[t2] + (lt - 1), reg_data_ss_anom_ens[[i_reg]][1, , lt], col = "black", type = type_sdate, 
         lwd = 1.5, ylim = ylim, xlim = range(yrs), cex.lab = pcexlab, 
         cex.axis = pcexax, xlab = "Start Date", 
         ylab = paste0("T (", vari_units, ")"), lty = 1)
  if (!is.null(recon)) {
    lines(yrs[t2] + (lt - 1), reg_recon_ss_anom_ens[[i_reg]][1, , lt], col = "grey", lwd = 1.5)
  }
    lines(yrs[t2] + (lt - 1), reg_hist_ss_anom_ens[[i_reg]][1, , lt], col = "blue", lwd = 1.5)
    lines(yrs[t2] + (lt - 1), reg_obs_ss_anom_ens[[i_reg]][1, , lt], col = "red", lwd = 1.5)
    lines(yrs + (lt - 1), reg_hist_ss_full_anom_ens[[i_reg]][1, , lt], col = "blue", lwd = 1.5, lty = 2)
    
  } else { #discrete sdate
    # create a frame without data first
    plot(yrs + (lt - 1), y = NULL, col = "black", type = type_sdate, lwd = 0.5, 
         ylim = ylim, xlim = range(yrs), cex.lab = pcexlab, 
         cex.axis = pcexax, xlab = "Start Date", 
         ylab = paste0("T (", vari_units, ")"), lty = 1)
    
    lines(yrs[t2] + (lt - 1), reg_data_ss_anom_ens[[i_reg]][1, , lt], col = 'black', type = type_sdate, lwd = 1.5)
  if (!is.null(recon)) {
    lines(yrs[t2] + (lt - 1), reg_recon_ss_anom_ens[[i_reg]][1, , lt], col = 'grey', type = type_sdate, lwd = 1.5)
  }
    lines(yrs[t2] + (lt - 1), reg_hist_ss_anom_ens[[i_reg]][1, , lt], col = 'blue', type = type_sdate, lwd = 1.5)
    lines(yrs[t2] + (lt - 1), reg_obs_ss_anom_ens[[i_reg]][1, , lt], col = 'red', type = type_sdate, lwd = 1.5)
    lines(yrs + (lt - 1), reg_hist_ss_full_anom_ens[[i_reg]][1, , lt], col = 'blue', type = 'l',, lwd = 1.5, lty = 2)
  }
  
  # Add region title
  title(main = reg[i_reg], adj = 0, cex.main = pcexmain, line = 0.6)
  # Decide legend position
  legend_pos <- ifelse(reg_data_ss_anom_ens[[i_reg]][1, 1, lt] > mean(ylim), "bottomleft", "topleft")
  if (is.na(legend_pos)) legend_pos <- 'topleft'

  if (!is.null(recon)) { 
    legend(legend_pos, 
           legend = c(exp, recon, 'hist', obs, 'hist_full'), 
           col = c("black", 'grey', "blue", "red", "blue"), 
           lty = c(1, 1, 1, 1, 2), bty = "n", cex = cex.legend)
  } else {
    legend(legend_pos,
           legend = c(exp, 'hist', obs, 'hist_full'), 
           col = c("black", "blue", "red", "blue"),
           lty = c(1, 1, 1, 2), bty = "n", cex = cex.legend)
  }
}

# Add common main title
mtext(paste0('Anomaly, leadtime ', lt, ', ', vari, ', ', exp, ', Init: ', init_mon),
      side = 3, line = -2, outer = TRUE, cex = cex.title)
mtext(info_tnm, side = 3, line = -3.5, outer = TRUE, cex = cex.sub_title)

if (savefile) dev.off()

if (INFO) print("==========PLOTTING TYPE II FINISHED.===========")


#===== Plotting: type III (Lead-time dependent climatology) =====

mfrow <- fun_nice_mfrow(reg)
if (savefile) {
  png(paste0(savefile_path, 'timeseries_figIII_', exp, '_', vari, '.png'),
      width = 500 * mfrow[2], height = 360 * mfrow[1])
} else {
  dev.new(width = 7 * 2, height = 3 * (mfrow[1] / mfrow[2]) * 2)
}
# increase upper mar
#new_mar <- c(par()$mar[1:2], par()$mar[3] * 1.5, par()$mar[4])
new_mar <- c(4.1, 4.1, 5.8, 2.1)
par(mfrow = mfrow, cex = 1, mar = new_mar)
# adjust space between axis labels & axis, tick labels & ticks
par(mgp = c(2, 0.5, 0))

pcexlab <- 1
pcexmain <- 1.2
pcexax <- 1
cex.legend <- 0.8
cex.title <- 1.6
cex.sub_title <- 1


for (i_reg in 1:length(reg)) {
  
  fyr <- c(1:dim(reg_clim_data[[i_reg]]$clim_exp)['time'])
  if (reg[i_reg] == 'Nino3.4') {
    xlabel <- 'Forecast Month'
  } else {
    xlabel <- 'Forecast Year'
  }

  if (!is.null(recon)) { 
  max_range <- range(range(reg_clim_data[[i_reg]]$clim_exp[1, ], na.rm = T),
                     range(reg_clim_recon[[i_reg]]$clim_exp[1, ], na.rm = T),
                     range(reg_clim_hist[[i_reg]]$clim_exp[1, ], na.rm = T),
                     range(reg_clim_data[[i_reg]]$clim_obs[1, ], na.rm = T),
                     range(reg_clim_hist_full[[i_reg]]$clim_exp[1, ], na.rm = T),
                     finite = T)
  } else {
  max_range <- range(range(reg_clim_data[[i_reg]]$clim_exp[1, ], na.rm = T),
                     range(reg_clim_hist[[i_reg]]$clim_exp[1, ], na.rm = T),
                     range(reg_clim_data[[i_reg]]$clim_obs[1, ], na.rm = T),
                     range(reg_clim_hist_full[[i_reg]]$clim_exp[1, ], na.rm = T),
                     finite = T)
  }
  enlarge_ratio <- 0.2
  ylim <- c(max_range[1] - diff(max_range) * enlarge_ratio, max_range[2] + diff(max_range) * enlarge_ratio)
  plot(fyr, reg_clim_data[[i_reg]]$clim_exp[1, ], col = "black", type = "l", lwd = 1,
       ylim = ylim, xlim = range(fyr), cex.lab = pcexlab, cex.axis = pcexax,
       xlab = xlabel, ylab = paste0("T (", vari_units, ")"))

  if (!is.null(recon)) { 
  lines(fyr, reg_clim_recon[[i_reg]]$clim_exp[1, ], col = 'grey', type = "l", lwd = 1)
  }
  lines(fyr, reg_clim_hist[[i_reg]]$clim_exp[1, ], col = "blue", type = "l", lwd = 1)
  lines(fyr, reg_clim_data[[i_reg]]$clim_obs[1, ], col = 'red', type = "l", lwd = 1)
  lines(fyr, reg_clim_hist_full[[i_reg]]$clim_exp[1, ], col = 'blue', type = 'l', lty = 2, lwd = 1)
  
  # Add region title
  title(main = reg[i_reg], adj = 0, cex.main = pcexmain, line = 0.6)
  # Decide legend position
  legend_y_pos <- ifelse(reg_clim_data[[i_reg]]$clim_exp[1, 1] > mean(ylim), "bottomleft", "topleft")
  if (is.na(legend_y_pos)) legend_y_pos <- 'topleft'

  if (!is.null(recon)) {
  legend(legend_y_pos, 
         legend = c(exp, recon, 'hist', obs, 'hist_full'), 
         col = c("black", "grey", "blue", "red", "blue"), 
         lty = c(1, 1, 1, 1, 2), bty = "n", cex = cex.legend) #, y.intersp = 0.7)
  } else {
  legend(legend_y_pos,
         legend = c(exp, 'hist', obs, 'hist_full'),
         col = c("black", "blue", "red", "blue"),
         lty = c(1, 1, 1, 2), bty = "n", cex = cex.legend) #, y.intersp = 0.7)
  }
}

# Add common main title
mtext(paste0('Climatology, ', vari, ', ', exp, ', Init: ', init_mon),
      side = 3, line = -2, outer = TRUE, cex = cex.title)
mtext(info_tnm, side = 3, line = -3.5, outer = TRUE, cex = cex.sub_title)

if (savefile) dev.off()

if (INFO) print("==========PLOTTING TYPE III FINISHED.===========")


#===== Plotting: type IV (ACC) =====

pcexlab <- 1
pcexmain <- 1.2
pcexax <- 1
cex.legend <- 0.8
cex.title <- 1.6
cex.sub_title <- 1

if (!is.null(recon)) {
  plot_i_total <- 1:2
} else {
  plot_i_total <- 1
}

for (plot_i in plot_i_total) {

  if (plot_i == 1) {
  #--- figIVa: exp-obs ---
    reg_acc_data_i <- reg_acc_data
    reg_acc_data_sig_i <- reg_acc_data_sig
    reg_sig_corr_i <- reg_sig_corr
    reg_acc_hist_i <- reg_acc_hist
    reg_acc_hist_sig_i <- reg_acc_hist_sig
    reg_acc_hist_full_i <- reg_acc_hist_full
    reg_acc_hist_full_sig_i <- reg_acc_hist_full_sig
  } else {
    reg_acc_data_i <- reg_acc_data_recon
    reg_acc_data_sig_i <- reg_acc_data_recon_sig
    reg_sig_corr_i <- reg_sig_corr_recon
    reg_acc_hist_i <- reg_acc_hist_recon
    reg_acc_hist_sig_i <- reg_acc_hist_recon_sig
    reg_acc_hist_full_i <- reg_acc_hist_recon_full
    reg_acc_hist_full_sig_i <- reg_acc_hist_recon_full_sig
  }


if (savefile) {
  png(paste0(savefile_path, 'timeseries_figIV', c('a', 'b')[plot_i], '_', exp, '_', vari, '.png'),
      width = 500 * mfrow[2], height = 360 * mfrow[1])
} else {
  dev.new(width = 7 * mfrow[2], height = 3 * mfrow[1])
}
# increase upper mar
#new_mar <- c(par()$mar[1:2], par()$mar[3] * 1.5, par()$mar[4])
new_mar <- c(4.1, 4.1, 5.8, 2.1)
par(mfrow = mfrow, cex = 1, mar = new_mar)
# adjust space between axis labels & axis, tick labels & ticks
par(mgp = c(2, 0.5, 0))

for (i_reg in 1:length(reg)) {
  
  fyr <- c(1:dim(reg_acc_data[[i_reg]]$corr)['time'])
  
  if (reg[i_reg] == 'Nino3.4') {
    xlabel <- 'Forecast Month'
  } else {
    xlabel <- 'Forecast Year'
  }

  #line exp-obs/recon
  plot(fyr, reg_acc_data_i[[i_reg]]$corr[1, 1, ], col = "black", type = "l", lwd = 1,
       ylim = c(-1, 1), xlim = range(fyr), cex.axis = pcexax,
       xlab = xlabel, ylab = "Cor", cex.lab = pcexlab)
#       xlab = "", ylab = "Cor", cex.lab = pcexlab)
  #sig1: pch1 = circle
  lines(fyr, reg_acc_data_i[[i_reg]]$corr[1, 1, ] * reg_acc_data_sig_i[[i_reg]], col = "black", type = "p", lwd = 1.0, pch = 1, cex = 1)
  #sig2: pch16 = filled circle
  lines(fyr, reg_acc_data_i[[i_reg]]$corr[1, 1, ] * reg_sig_corr_i[[i_reg]], col = "black", type = "p", lwd = 1.0, pch = 16, cex = 1)
  #line hist-obs/recon:
  lines(fyr, reg_acc_hist_i[[i_reg]]$corr[1, 1, ], col = 'blue', type = "l", lwd = 1)
  lines(fyr, reg_acc_hist_i[[i_reg]]$corr[1, 1, ] * reg_acc_hist_sig_i[[i_reg]], col = "blue", type = "p", lwd = 1.0, pch = 1, cex = 1)
  #line hist-obs/recon_full:
  lines(fyr, reg_acc_hist_full_i[[i_reg]]$corr[1, 1, ], col = 'blue', type = "l", lty = 2, lwd = 1)
  lines(fyr, reg_acc_hist_full_i[[i_reg]]$corr[1, 1, ] * reg_acc_hist_full_sig_i[[i_reg]], col = "blue", type = "p", lwd = 1.0, pch = 1, cex = 1)
  
  # Add region title
  title(main = reg[i_reg], adj = 0, cex.main = pcexmain, line = 0.6)
#  title(xlab = xlabel, line = 4)

  # Decide legend position
  legend_pos <- tryCatch({
    if (reg_acc_data_i[[i_reg]]$corr[1, 1, 1] > -0.3 & reg_acc_hist_i[[i_reg]]$corr[1, 1, 1] > -0.3 & reg_acc_hist_full_i[[i_reg]]$corr[1, 1, 1] > -0.3) {
      'bottomleft'
    } else if (reg_acc_data_i[[i_reg]]$corr[1, 1, 1] < 0.3 & reg_acc_hist_i[[i_reg]]$corr[1, 1, 1] < 0.3 & reg_acc_hist_full_i[[i_reg]]$corr[1, 1, 1] < 0.3) {
      'topleft'
    } else {
      'left'
    }
  },
  error = function(x) {
    return('left')
  })

  # Create legend
  legend(legend_pos,
         legend = c(exp, 'hist', 'hist_full'), 
         col = c("black", "blue", "blue"),
         lty = c(1, 1, 2), bty = "n", cex = cex.legend)
  
} # region for loop

# Add common main title
if (plot_i == 1) {
  mtext(paste0('ACC w/', obs, ', ', vari, ', ', exp, ', Init: ', init_mon),
        side = 3, line = -2, outer = TRUE, cex = cex.title)
} else {
  mtext(paste0('ACC w/', recon, ', ', vari, ', ', exp, ', Init: ', init_mon),
        side = 3, line = -2, outer = TRUE, cex = cex.title)
}
mtext(info_tnm, side = 3, line = -3.5, outer = TRUE, cex = cex.sub_title)

if (savefile) dev.off()

}  # plot_i for loop

if (INFO) print("==========PLOTTING TYPE IV FINISHED.===========")

if (INFO) print("==========FINISHED SUCCESSFULLY.================")

if (Write_to_log) sink()
